{*******************************************************}
{                                                       }
{       RichView                                        }
{       Demo: using ASpell component                    }
{       with TRichViewEdit                              }
{                                                       }
{       RichView: Copyright (c) Sergey Tkachenko        }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{       ASpell : Copyright (c) Kevin Atkinson           }
{       http://aspell.net                               }
{                                                       }
{*******************************************************}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, RVStyle, RVScroll, RichView, RVEdit, StdCtrls, RVWordEnum,
  Menus, RVASpell, ExtCtrls;

type
  TForm1 = class(TForm)
    RichViewEdit1: TRichViewEdit;
    RVStyle1: TRVStyle;
    Button1: TButton;
    PopupMenu1: TPopupMenu;
    RVASpell1: TRVASpell;
    ComboBox1: TComboBox;
    Label1: TLabel;
    RadioGroup1: TRadioGroup;
    procedure FormCreate(Sender: TObject);
    procedure RichViewEdit1SpellingCheck(Sender: TCustomRichView;
      const AWord: String; StyleNo: Integer; var Misspelled: Boolean);
    procedure Button1Click(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure RVASpell1SpellFormAction(Sender: TRVASpell; const AWord,
      AReplaceTo: String; Action: TRVASpellFormAction);
    procedure ComboBox1Click(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
  private
    { Private declarations }
    procedure PopupMenuItemClick(Sender: TObject);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  RichViewEdit1.Clear;
  RichViewEdit1.AddNLTag(
    'Speling contest - a kontest in wich you are elimineited if you fale to spel a vord corectly.',
    0, 0, 0);
  RichViewEdit1.Format;
  RichViewEdit1.StartLiveSpelling;
  PopupMenu1.AutoHotkeys := maManual;

  ComboBox1.Items := RVASpell1.Dictionaries;
  ComboBox1.ItemIndex := RVASpell1.DictIndex;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  RVASpell1.Check(RichViewEdit1, rvesFromStart);
end;

procedure TForm1.RichViewEdit1SpellingCheck(Sender: TCustomRichView;
  const AWord: String; StyleNo: Integer; var Misspelled: Boolean);
begin
  Misspelled := RVASpell1.IsMisspelled(AWord, StyleNo, Sender);
end;

procedure TForm1.RVASpell1SpellFormAction(Sender: TRVASpell; const AWord,
  AReplaceTo: String; Action: TRVASpellFormAction);
begin
  if Action in [rvasIgnoreAll, rvasChangeAll, rvasAdd] then
    RichViewEdit1.LiveSpellingValidateWord(AWord);
end;

procedure TForm1.PopupMenu1Popup(Sender: TObject);
var wrd: String;
    stl, i: Integer;
    sl: TStringList;
    mi: TMenuItem;
begin
  while PopupMenu1.Items.Count>0 do
    PopupMenu1.Items[0].Free;
  if RichViewEdit1.GetCurrentMisspelling(True, wrd, stl) then begin
    sl := TStringList.Create;
    try
      RVASpell1.SuggestWords(wrd, RichViewEdit1.CurTextStyleNo, sl, RichViewEdit1);
      for i := 0 to sl.Count-1 do begin
        mi := TMenuItem.Create(PopupMenu1);
        mi.Caption := sl[i];
        mi.OnClick := PopupMenuItemClick;
        PopupMenu1.Items.Add(mi);
      end;
      if sl.Count=0 then begin
        mi := TMenuItem.Create(PopupMenu1);
        mi.Caption := '(no suggestions)';
        mi.Enabled := False;
        PopupMenu1.Items.Add(mi);
      end;
    finally
      sl.Free;
    end;
  end;
end;

procedure TForm1.PopupMenuItemClick(Sender: TObject);
begin
  RichViewEdit1.InsertText((Sender as TMenuItem).Caption);
end;

procedure TForm1.ComboBox1Click(Sender: TObject);
begin
  RichViewEdit1.ClearLiveSpellingResults;
  RVASpell1.DictIndex := ComboBox1.ItemIndex;
  RichViewEdit1.StartLiveSpelling;
end;

procedure TForm1.RadioGroup1Click(Sender: TObject);
begin
  RVASpell1.SpellFormStyle := TRVASpellFormStyle(RadioGroup1.ItemIndex);
end;

end.