{*******************************************************}
{                                                       }
{       RichView                                        }
{       Demo: using ASpell component                    }
{       with RichViewActions                            }
{                                                       }
{       RichView: Copyright (c) Sergey Tkachenko        }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{       ASpell : Copyright (c) Kevin Atkinson           }
{       http://aspell.net                               }
{                                                       }
{*******************************************************}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, RVStyle, RVScroll, RichView, RVEdit, StdCtrls, RVWordEnum,
  Menus, RichViewActions, ActnList, RVASpell, ExtCtrls;

type
  TForm1 = class(TForm)
    RichViewEdit1: TRichViewEdit;
    RVStyle1: TRVStyle;
    Button1: TButton;
    RVAPopupMenu1: TRVAPopupMenu;
    ActionList1: TActionList;
    Label1: TLabel;
    ComboBox1: TComboBox;
    RVASpell1: TRVASpell;
    RadioGroup1: TRadioGroup;
    procedure FormCreate(Sender: TObject);
    procedure RichViewEdit1SpellingCheck(Sender: TCustomRichView;
      const AWord: String; StyleNo: Integer; var Misspelled: Boolean);
    procedure Button1Click(Sender: TObject);
    procedure RVAPopupMenu1LiveSpellGetSuggestions(Sender: TRVAPopupMenu;
      Edit: TCustomRichViewEdit; const Word: String; StyleNo: Integer;
      Suggestions: TStrings);
    procedure RVAPopupMenu1LiveSpellAdd(Sender: TRVAPopupMenu;
      Edit: TCustomRichViewEdit; const Word: String; StyleNo: Integer);
    procedure RVAPopupMenu1LiveSpellIgnoreAll(Sender: TRVAPopupMenu;
      Edit: TCustomRichViewEdit; const Word: String; StyleNo: Integer);
    procedure RVASpell1SpellFormAction(Sender: TRVASpell; const AWord,
      AReplaceTo: String; Action: TRVASpellFormAction);
    procedure ComboBox1Click(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  RichViewEdit1.Clear;
  RichViewEdit1.AddNLTag(
    'Speling contest - a kontest in wich you are elimineited if you fale to spel a vord corectly.',
    0, 0, 0);
  RichViewEdit1.Format;
  RichViewEdit1.StartLiveSpelling;
  RVA_LocalizeForm(Self); // assigning captions to "Ignore All" and "Add" menu items

  ComboBox1.Items := RVASpell1.Dictionaries;
  ComboBox1.ItemIndex := RVASpell1.DictIndex;  
end;
{ Starting a traditional (with dialog) spelling }
procedure TForm1.Button1Click(Sender: TObject);
begin
  RVASpell1.Check(RichViewEdit1, rvesFromStart);
end;
{ Live spelling event }
procedure TForm1.RichViewEdit1SpellingCheck(Sender: TCustomRichView;
  const AWord: String; StyleNo: Integer; var Misspelled: Boolean);
begin
  Misspelled := RVASpell1.IsMisspelled(AWord, StyleNo, Sender);
end;
{ Called when button in the spelling dialog is clicked }
procedure TForm1.RVASpell1SpellFormAction(Sender: TRVASpell; const AWord,
  AReplaceTo: String; Action: TRVASpellFormAction);
begin
  if Action in [rvasIgnoreAll, rvasChangeAll] then
    RichViewEdit1.LiveSpellingValidateWord(AWord);
end;
{ List of suggestions for menu }
procedure TForm1.RVAPopupMenu1LiveSpellGetSuggestions(
  Sender: TRVAPopupMenu; Edit: TCustomRichViewEdit; const Word: String;
  StyleNo: Integer; Suggestions: TStrings);
begin
  RVASpell1.SuggestWords(Word, StyleNo, Suggestions, Edit);
end;
{ Add to dictionary }
procedure TForm1.RVAPopupMenu1LiveSpellAdd(Sender: TRVAPopupMenu;
  Edit: TCustomRichViewEdit; const Word: String; StyleNo: Integer);
begin
  RVASpell1.AddToDictionary(Word, StyleNo, Edit);
  // RichViewEdit1.LiveSpellingValidateWord is called by the menu
end;
{ Add to ignore list }
procedure TForm1.RVAPopupMenu1LiveSpellIgnoreAll(Sender: TRVAPopupMenu;
  Edit: TCustomRichViewEdit; const Word: String; StyleNo: Integer);
begin
  RVASpell1.AddToIgnoreList(Word, StyleNo, Edit);
  // RichViewEdit1.LiveSpellingValidateWord is called by the menu  
end;
{ Changing dictionary }
procedure TForm1.ComboBox1Click(Sender: TObject);
begin
  RichViewEdit1.ClearLiveSpellingResults;
  RVASpell1.DictIndex := ComboBox1.ItemIndex;
  RichViewEdit1.StartLiveSpelling;
end;

procedure TForm1.RadioGroup1Click(Sender: TObject);
begin
  RVASpell1.SpellFormStyle := TRVASpellFormStyle(RadioGroup1.ItemIndex);
end;

end.