
{*******************************************************}
{                                                       }
{       Demo for using ExpressSpellChecker and          }
{       TRichViewEdit.                                  }
{                                                       }
{       TRichView (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{       ExpressSpellChecker (c) Developer Express Inc.  }
{       http://www.devexpress.com                       }
{                                                       }
{*******************************************************}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, RVStyle, StdCtrls, RVScroll, RichView, RVEdit, RVDXSpell,
  dxSpellChecker, dxISpellDecompressor, RVUni, cxLookAndFeelPainters,
  cxControls, cxContainer, cxEdit, cxGroupBox, cxRadioGroup, ExtCtrls, cxButtons,
  RVTypes;

type
  TForm1 = class(TForm)
    RichViewEdit1: TRichViewEdit;
    RVStyle1: TRVStyle;
    PopupMenu1: TPopupMenu;
    cxRadioGroup1: TcxRadioGroup;
    btnSpell: TcxButton;
    procedure FormCreate(Sender: TObject);
    procedure cxRadioGroup1Click(Sender: TObject);
    procedure btnSpellClick(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure RichViewEdit1SpellingCheck(Sender: TCustomRichView;
      const AWord: string; StyleNo: Integer; var Misspelled: Boolean);
  private
    { Private declarations }
    procedure PopupMenuItemClick(Sender: TObject);
    procedure RVdxSpellChecker1SpellFormAction(Sender: TRvDxSpellChecker;
      const AWord, AReplaceTo: TRVUnicodeString; Action: TRVdxSpellFormAction;
      AReplacingItemText: Boolean);
  public
    { Public declarations }
    RVdxSpellChecker1: TRVdxSpellChecker;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{------------------------------------------------------------------------------}
procedure TForm1.FormCreate(Sender: TObject);
var
  ADictionaryItem: TdxSpellCheckerDictionaryItem;
begin
  // Creating a sample document
  RichViewEdit1.Clear;
  RichViewEdit1.AddNLTag(
    'Speling contest - a kontest in wich you are elimineited if you fale to spel a vord corectly.',
    0,0, 0);
  RichViewEdit1.Format;

  PopupMenu1.AutoHotkeys := maManual;
  // Creating a spell checker component
  RVdxSpellChecker1 := TRVdxSpellChecker.Create(Self);
  RVdxSpellChecker1.OnSpellFormAction := RVdxSpellChecker1SpellFormAction;
  // Adding ISpell English dictionary
  ADictionaryItem := RVdxSpellChecker1.DictionaryItems.Add;
  ADictionaryItem.DictionaryTypeClass := TdxISpellDictionary;
  with TdxISpellDictionary(ADictionaryItem.DictionaryType) do begin
    DictionaryPath := 'ispell-english\american.xlg';
    GrammarPath := 'ispell-english\english.aff';
    CodePage := 1252; // Western languages
    Language := 1033; // English (US)
  end;
  // Adding user dictionary
  ADictionaryItem := RVdxSpellChecker1.DictionaryItems.Add;
  ADictionaryItem.DictionaryTypeClass := TdxUserSpellCheckerDictionary;
  with TdxUserSpellCheckerDictionary(ADictionaryItem.DictionaryType) do begin
    DictionaryPath := 'custom.dic';
  end;
  RVdxSpellChecker1.LoadDictionaries;
  RVdxSpellChecker1.UpdateRules;
  RichViewEdit1.StartLiveSpelling;
end;
{------------------------------------------------------------------------------}
// Performing spell checking
procedure TForm1.btnSpellClick(Sender: TObject);
begin
  RVdxSpellChecker1.Check(RichViewEdit1);
end;
{------------------------------------------------------------------------------}
// Changing dialog style
procedure TForm1.cxRadioGroup1Click(Sender: TObject);
begin
  RVdxSpellChecker1.SpellingFormType :=
    TdxSpellCheckerSpellingFormType(cxRadioGroup1.ItemIndex);
end;
{------------------------------------------------------------------------------}
// Adding suggestions for misspelled word in the popup menu, before it is shown
procedure TForm1.PopupMenu1Popup(Sender: TObject);
var wrd: String;
    stl, i: Integer;
    sl: TStringList;
    mi: TMenuItem;
begin
  while PopupMenu1.Items.Count>0 do
    PopupMenu1.Items[0].Free;
  if RichViewEdit1.GetCurrentMisspelling(True, wrd, stl) then begin
    sl := TStringList.Create;
    try
      RVdxSpellChecker1.SuggestWords(wrd, sl);
      for i := 0 to sl.Count-1 do begin
        mi := TMenuItem.Create(PopupMenu1);
        mi.Caption := sl[i];
        mi.OnClick := PopupMenuItemClick;
        PopupMenu1.Items.Add(mi);
      end;
      if sl.Count=0 then begin
        mi := TMenuItem.Create(PopupMenu1);
        mi.Caption := '(no suggestions)';
        mi.Enabled := False;
        PopupMenu1.Items.Add(mi);
      end;
    finally
      sl.Free;
    end;
  end;
end;
{------------------------------------------------------------------------------}
// Clicking on menu item replaces the selection with the correction
procedure TForm1.PopupMenuItemClick(Sender: TObject);
begin
  RichViewEdit1.InsertText((Sender as TMenuItem).Caption);
end;
{------------------------------------------------------------------------------}
// Checking word on live spelling
procedure TForm1.RichViewEdit1SpellingCheck(Sender: TCustomRichView;
  const AWord: string; StyleNo: Integer; var Misspelled: Boolean);
begin
  Misspelled := RVdxSpellChecker1.IsMisspelled(AWord);
end;
{------------------------------------------------------------------------------}
// Validating word when IgnoreAll or Add commands were chosen in the dictionary
procedure TForm1.RVdxSpellChecker1SpellFormAction(Sender: TRvDxSpellChecker;
  const AWord, AReplaceTo: TRVUnicodeString; Action: TRVdxSpellFormAction;
  AReplacingItemText: Boolean);
begin
  if Action in [rvdxIgnoreAll, rvdxAdd] then
    RichViewEdit1.LiveSpellingValidateWord(AWord);
end;

end.
