unit Unit1;

interface

{$I RV_Defs.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, RVStyle, StdCtrls, RVScroll, RichView, RVEdit, RVWordEnum,
  RVHunSpell, RVTypes, RVUni, ExtCtrls;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    RichViewEdit1: TRichViewEdit;
    Button1: TButton;
    RVStyle1: TRVStyle;
    PopupMenu1: TPopupMenu;
    ListBox1: TListBox;
    RVHunSpell1: TRVHunSpell;
    RadioGroup1: TRadioGroup;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure RichViewEdit1SpellingCheck(Sender: TCustomRichView;
      const AWord: string; StyleNo: Integer; var Misspelled: Boolean);
    procedure RVHunSpell1SpellFormAction(Sender: TRVHunSpell; const AWord,
      AReplaceTo: TRVUnicodeString; Action: TRVHunSpellFormAction);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
  private
    procedure PopupMenuItemClick(Sender: TObject);
    function GetUnicode(const s: String; StyleNo: Integer): TRVUnicodeString;    
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}


procedure TForm1.FormCreate(Sender: TObject);
begin
  RichViewEdit1.Clear;
  RichViewEdit1.AddNLTag(
    'Speling contest - a kontest in wich you are elimineited if you fale to spel a vord corectly.',
    0, 0, 0);
  RichViewEdit1.Format;
  PopupMenu1.AutoHotkeys := maManual;
  // This demo assumes that dictionaries are in the application directory
  RVHunSpell1.Dictionaries.Clear;
  RVHunSpell1.LoadAllDictionaries;
  ListBox1.Items.Assign(RVHunSpell1.Dictionaries);
  RichViewEdit1.StartLiveSpelling;
end;
{ Converting s to Unicode }
function TForm1.GetUnicode(const s: String; StyleNo: Integer): TRVUnicodeString;
begin
  {$IFNDEF RICHVIEWDEF2009}
  // Delphi 4-2007: converting s to Unicode basing on the text style's Charset
  Result := RVU_RawUnicodeToWideString(RVU_AnsiToUnicode(
    RichViewEdit1.RVData.GetStyleCodePage(StyleNo), s));
  {$ELSE}
  // Delphi 2009+: s is already Unicode
  Result := s;
  {$ENDIF}
end;
{ Starting spelling }
procedure TForm1.Button1Click(Sender: TObject);
begin
  RVHunSpell1.Check(RichViewEdit1, rvesFromStart);
end;
{ Live spelling event }
procedure TForm1.RichViewEdit1SpellingCheck(Sender: TCustomRichView;
  const AWord: string; StyleNo: Integer; var Misspelled: Boolean);
begin
  Misspelled := not RVHunSpell1.Spell(GetUnicode(AWord, StyleNo));
end;

procedure TForm1.RVHunSpell1SpellFormAction(Sender: TRVHunSpell; const AWord,
  AReplaceTo: TRVUnicodeString; Action: TRVHunSpellFormAction);
begin
  if Action in [rvhsIgnoreAll, rvhsAdd] then
    RichViewEdit1.LiveSpellingValidateWord(AWord);
end;

procedure TForm1.PopupMenu1Popup(Sender: TObject);
var wrd: String;
    stl, i: Integer;
    sl: TStringList;
    mi: TMenuItem;
begin
  while PopupMenu1.Items.Count>0 do
    PopupMenu1.Items[0].Free;
  if RichViewEdit1.GetCurrentMisspelling(True, wrd, stl) then begin
    sl := TStringList.Create;
    try
      RVHunSpell1.Suggest(GetUnicode(wrd, stl), sl);
      for i := 0 to sl.Count-1 do begin
        mi := TMenuItem.Create(PopupMenu1);
        mi.Caption := sl[i];
        mi.OnClick := PopupMenuItemClick;
        PopupMenu1.Items.Add(mi);
      end;
      if sl.Count=0 then begin
        mi := TMenuItem.Create(PopupMenu1);
        mi.Caption := '(no suggestions)';
        mi.Enabled := False;
        PopupMenu1.Items.Add(mi);
      end;
    finally
      sl.Free;
    end;
  end;
end;

procedure TForm1.PopupMenuItemClick(Sender: TObject);
begin
  RichViewEdit1.InsertText((Sender as TMenuItem).Caption);
end;

procedure TForm1.RadioGroup1Click(Sender: TObject);
begin
  RVHunSpell1.SpellFormStyle := TRVHunSpellFormStyle(RadioGroup1.ItemIndex);
end;

end.
