{*******************************************************}
{                                                       }
{       RichView                                        }
{       Demo: using Polar Spell Checker ActiveX         }
{       with TRichViewEdit                              }
{                                                       }
{       RichView: Copyright (c) Sergey Tkachenko        }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{       Polar SpellChecker Component :                  }
{        Copyright (c) Polar                            }
{       http://www.polarsoftware.com/                   }
{                                                       }
{*******************************************************}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, RVStyle, RVScroll, RichView, RVEdit, StdCtrls, RVWordEnum,
  Menus, ExtCtrls, OleCtrls, SPELLCHECKERLib_TLB, RVPolarSpell, RVUni, RVTypes;

{$I RV_Defs.inc}

type
  TForm1 = class(TForm)
    RichViewEdit1: TRichViewEdit;
    RVStyle1: TRVStyle;
    Button1: TButton;
    PopupMenu1: TPopupMenu;
    RVPolarSpellChecker1: TRVPolarSpellChecker;
    procedure FormCreate(Sender: TObject);
    procedure RichViewEdit1SpellingCheck(Sender: TCustomRichView;
      const AWord: String; StyleNo: Integer; var Misspelled: Boolean);
    procedure Button1Click(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure RVPolarSpellChecker1WordAdded(ASender: TObject;
      const Object_: ISpellChecker; const Word,
      CustomDictionary: WideString; AlreadyExists: WordBool);
    procedure RVPolarSpellChecker1WordAddedToIgnore(ASender: TObject;
      const Object_: ISpellChecker; const Word: WideString;
      AlreadyExists: WordBool);
  private
    { Private declarations }
    procedure PopupMenuItemClick(Sender: TObject);
    function GetUnicode(const s: String;
      StyleNo: Integer): TRVUnicodeString;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
var CustomDicFile: String;
begin
  RVPolarSpellChecker1.OpenDictionary('d:\VCL\Polar Component Suite\Polar SpellChecker Component 5\Dictionaries\SpellUS.lex');
  CustomDicFile := ExtractFilePath(Application.ExeName)+'custom.dic';
  if not FileExists(CustomDicFile) then
    RVPolarSpellChecker1.CreateCustomDictionary(CustomDicFile);
  RVPolarSpellChecker1.OpenCustomDictionary(CustomDicFile);
  RichViewEdit1.Clear;
  RichViewEdit1.AddNLTag(
    'Speling contest - a kontest in wich you are elimineited if you fale to spel a vord corectly.',
    0, 0, 0);
  RichViewEdit1.Format;
  RichViewEdit1.StartLiveSpelling;
  PopupMenu1.AutoHotkeys := maManual;
end;

// Converting s to Unicode
function TForm1.GetUnicode(const s: String; StyleNo: Integer): TRVUnicodeString;
begin
  {$IFNDEF RICHVIEWDEF2009} // defined in RV_Defs.inc
  // Delphi 4-2007: converting s to Unicode basing on the text style's Charset
  Result := RVU_RawUnicodeToWideString(RVU_AnsiToUnicode(
    RichViewEdit1.RVData.GetStyleCodePage(StyleNo), s));
  {$ELSE}
  // Delphi 2009+: s is already Unicode
  Result := s;
  {$ENDIF}
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  RVPolarSpellChecker1.CheckRichViewEdit(RichViewEdit1, rvesFromCursor);
end;

procedure TForm1.RichViewEdit1SpellingCheck(Sender: TCustomRichView;
  const AWord: String; StyleNo: Integer; var Misspelled: Boolean);
var Word: TRVUnicodeString;
begin
  Word := GetUnicode(AWord, StyleNo);
  Misspelled := not RVPolarSpellChecker1.IsWordIgnored(Word) and
    not RVPolarSpellChecker1.DoesWordExist(Word)
end;

procedure TForm1.PopupMenu1Popup(Sender: TObject);
var Word: String;
    WordW: TRVUnicodeString;
    StyleNo, i, Count: Integer;

    mi: TMenuItem;
begin
  while PopupMenu1.Items.Count>0 do
    PopupMenu1.Items[0].Free;
  if RichViewEdit1.GetCurrentMisspelling(True, Word, StyleNo) then begin
    WordW := GetUnicode(Word, StyleNo);
    Count := RVPolarSpellChecker1.GetSuggestionCount(WordW);
    for i := 1 to Count do begin
      mi := TMenuItem.Create(PopupMenu1);
      mi.Caption := RVPolarSpellChecker1.GetSuggestion(WordW, i);
      mi.OnClick := PopupMenuItemClick;
      PopupMenu1.Items.Add(mi);
    end;
    if Count=0 then begin
      mi := TMenuItem.Create(PopupMenu1);
      mi.Caption := '(no suggestions)';
      mi.Enabled := False;
      PopupMenu1.Items.Add(mi);
    end;
  end;
end;

procedure TForm1.PopupMenuItemClick(Sender: TObject);
begin
  RichViewEdit1.InsertText((Sender as TMenuItem).Caption);
end;

// This event occurs when Word is added to a custom dictionary
procedure TForm1.RVPolarSpellChecker1WordAdded(ASender: TObject;
  const Object_: ISpellChecker; const Word, CustomDictionary: WideString;
  AlreadyExists: WordBool);
begin
  if not AlreadyExists then
    RichViewEdit1.LiveSpellingValidateWord(Word);
end;
// This event occurs when Word is added to IgnoreAll list
procedure TForm1.RVPolarSpellChecker1WordAddedToIgnore(ASender: TObject;
  const Object_: ISpellChecker; const Word: WideString;
  AlreadyExists: WordBool);
begin
  if not AlreadyExists then
    RichViewEdit1.LiveSpellingValidateWord(Word);
end;

end.
