{*******************************************************}
{                                                       }
{       RichView                                        }
{       Demo: using Polar Spell Checker ActiveX         }
{       with RichViewActions                            }
{                                                       }
{       RichView: Copyright (c) Sergey Tkachenko        }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{       Polar SpellChecker Component :                  }
{        Copyright (c) Polar                            }
{       http://www.polarsoftware.com/                   }
{                                                       }
{*******************************************************}


unit Unit1;

{$I RV_Defs.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, RVStyle, RVScroll, RichView, RVEdit, StdCtrls, RVWordEnum,
  Menus, RichViewActions, ActnList, RVASpell, ExtCtrls, OleCtrls,
  SPELLCHECKERLib_TLB, RVPolarSpell, RVUni, RVTypes;

type
  TForm1 = class(TForm)
    RichViewEdit1: TRichViewEdit;
    RVStyle1: TRVStyle;
    Button1: TButton;
    RVAPopupMenu1: TRVAPopupMenu;
    ActionList1: TActionList;
    RVPolarSpellChecker1: TRVPolarSpellChecker;
    Button2: TButton;
    procedure FormCreate(Sender: TObject);
    procedure RichViewEdit1SpellingCheck(Sender: TCustomRichView;
      const AWord: String; StyleNo: Integer; var Misspelled: Boolean);
    procedure Button1Click(Sender: TObject);
    procedure RVAPopupMenu1LiveSpellGetSuggestions(Sender: TRVAPopupMenu;
      Edit: TCustomRichViewEdit; const Word: String; StyleNo: Integer;
      Suggestions: TStrings);
    procedure RVAPopupMenu1LiveSpellAdd(Sender: TRVAPopupMenu;
      Edit: TCustomRichViewEdit; const Word: String; StyleNo: Integer);
    procedure RVAPopupMenu1LiveSpellIgnoreAll(Sender: TRVAPopupMenu;
      Edit: TCustomRichViewEdit; const Word: String; StyleNo: Integer);
    procedure RVPolarSpellChecker1WordAdded(ASender: TObject;
      const Object_: ISpellChecker; const Word,
      CustomDictionary: WideString; AlreadyExists: WordBool);
    procedure RVPolarSpellChecker1WordAddedToIgnore(ASender: TObject;
      const Object_: ISpellChecker; const Word: WideString;
      AlreadyExists: WordBool);
    procedure RichViewEdit1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button2Click(Sender: TObject);
  private
    function GetUnicode(const s: String; StyleNo: Integer): TRVUnicodeString;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
var FileName: String;
begin
  RVPolarSpellChecker1.OpenDictionary('d:\VCL\Polar Component Suite\Polar SpellChecker Component 5\Dictionaries\SpellUS.lex');
  FileName := ExtractFilePath(Application.ExeName)+'custom.dic';
  if not FileExists(FileName) then
    RVPolarSpellChecker1.CreateCustomDictionary(FileName);
  RVPolarSpellChecker1.OpenCustomDictionary(FileName);
  FileName := ExtractFilePath(Application.ExeName)+'autocorrect.act';
  if FileExists(FileName) then
    RVPolarSpellChecker1.InitializeAutoCorrection(FileName);
  RichViewEdit1.Clear;
  RichViewEdit1.AddNLTag(
    'Speling contest - a kontest in wich you are elimineited if you fale to spel a vord corectly.',
    0, 0, 0);
  RichViewEdit1.Format;
  RichViewEdit1.StartLiveSpelling;
  RVA_LocalizeForm(Self); // assigning captions to "Ignore All" and "Add" menu items
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
var FileName: String;
begin
  FileName := ExtractFilePath(Application.ExeName)+'autocorrect.act';
  RVPolarSpellChecker1.SaveInitializationFile(FileName);
end;
{ Converting s to Unicode }
function TForm1.GetUnicode(const s: String; StyleNo: Integer): TRVUnicodeString;
begin
  {$IFNDEF RICHVIEWDEF2009} // defined in RV_Defs.inc
  // Delphi 4-2007: converting s to Unicode basing on the text style's Charset
  Result := RVU_RawUnicodeToWideString(RVU_AnsiToUnicode(
    RichViewEdit1.RVData.GetStyleCodePage(StyleNo), s));
  {$ELSE}
  // Delphi 2009+: s is already Unicode
  Result := s;
  {$ENDIF}
end;
{ Starting a traditional (with dialog) spelling }
procedure TForm1.Button1Click(Sender: TObject);
begin
  RVPolarSpellChecker1.CheckRichViewEdit(RichViewEdit1, rvesFromCursor);
end;
{ Live spelling event }
procedure TForm1.RichViewEdit1SpellingCheck(Sender: TCustomRichView;
  const AWord: String; StyleNo: Integer; var Misspelled: Boolean);
var Word: TRVUnicodeString;
begin
  Word := GetUnicode(AWord, StyleNo);
  Misspelled := not RVPolarSpellChecker1.IsWordIgnored(Word) and
    not RVPolarSpellChecker1.DoesWordExist(Word)
end;
{ This event occurs when Word is added to a custom dictionary }
procedure TForm1.RVPolarSpellChecker1WordAdded(ASender: TObject;
  const Object_: ISpellChecker; const Word, CustomDictionary: WideString;
  AlreadyExists: WordBool);
begin
  if not AlreadyExists then
    RichViewEdit1.LiveSpellingValidateWord(Word);
end;
{ This event occurs when Word is added to IgnoreAll list }
procedure TForm1.RVPolarSpellChecker1WordAddedToIgnore(ASender: TObject;
  const Object_: ISpellChecker; const Word: WideString;
  AlreadyExists: WordBool);
begin
  if not AlreadyExists then
    RichViewEdit1.LiveSpellingValidateWord(Word);
end;
{ List of suggestions for menu }
procedure TForm1.RVAPopupMenu1LiveSpellGetSuggestions(
  Sender: TRVAPopupMenu; Edit: TCustomRichViewEdit; const Word: String;
  StyleNo: Integer; Suggestions: TStrings);
var WordW: TRVUnicodeString;
    i, Count: Integer;
begin
  WordW := GetUnicode(Word, StyleNo);
  Count := RVPolarSpellChecker1.GetSuggestionCount(WordW);
  Suggestions.Clear;
  for i := 1 to Count do
    Suggestions.Add(RVPolarSpellChecker1.GetSuggestion(WordW, i));
end;
{ Add to dictionary }
procedure TForm1.RVAPopupMenu1LiveSpellAdd(Sender: TRVAPopupMenu;
  Edit: TCustomRichViewEdit; const Word: String; StyleNo: Integer);
begin
  if RVPolarSpellChecker1.ActiveCustomDictionaryPath='' then
    Beep
  else
    RVPolarSpellChecker1.AddWord(GetUnicode(Word, StyleNo));
  // RichViewEdit1.LiveSpellingValidateWord is called by the menu
end;
{ Add to ignore list }
procedure TForm1.RVAPopupMenu1LiveSpellIgnoreAll(Sender: TRVAPopupMenu;
  Edit: TCustomRichViewEdit; const Word: String; StyleNo: Integer);
begin
  RVPolarSpellChecker1.AddToIgnoreAll(GetUnicode(Word, StyleNo));
  // RichViewEdit1.LiveSpellingValidateWord is called by the menu
end;
{ Autocorrection }
procedure TForm1.RichViewEdit1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key in [VK_SPACE, VK_RETURN, VK_TAB] then
    RVPolarSpellChecker1.AutoCorrectRichViewEdit(RichViewEdit1);
end;
{ Autocorrection options }
procedure TForm1.Button2Click(Sender: TObject);
begin
  // Currently, only DUal caps and a list of words to replace supported
  RVPolarSpellChecker1.AutoCorrectOptionsDlg;
end;

end.