
{*******************************************************}
{                                                       }
{       TfrmRVDXSpellCheckerCustomForm - basic form for }
{       checking  spelling in TRichViewEdit controls    }
{       using  ExpressSpellChecker                      }
{                                                       }
{       TRichView (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{       ExpressSpellChecker (c) Developer Express Inc.  }
{       http://www.devexpress.com                       }
{                                                       }
{*******************************************************}

{  This form is not used directly.
   The following forms are inherited from it:
   - TfrmRVDXSpellCheckerOutlookForm (RVDXSpellOutlookForm.pas)
   - TfrmRVDXSpellCheckerWordForm (RVDXSpellWordForm.pas)
}

unit RVDXSpellCustomForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, cxLookAndFeelPainters, StdCtrls, dxCore,
  cxButtons, cxListBox, cxControls, cxContainer, cxEdit, cxTextEdit,
  dxSpellCheckerDialogs, dxSpellChecker, cxLookAndFeels,
  RVTypes, RVStyle, cxGraphics;

type
  TfrmRVDXSpellCheckerCustomForm = class(TForm)
    lbMisspelled: TLabel;
    lbSuggestions: TLabel;
    btnIgnore: TcxButton;
    btnChange: TcxButton;
    btnAdd: TcxButton;
    btnIgnoreAll: TcxButton;
    btnChangeAll: TcxButton;
    btnOptions: TcxButton;
    btnCancel: TcxButton;
    procedure FormCreate(Sender: TObject);
    procedure btnOptionsClick(Sender: TObject);
  private
    { Private declarations }
  protected
    FChecker: TdxSpellChecker;
    FMisspelledWord: String;
    FHasSuggestions: Boolean;
    FTextStyle: TFontInfo;
    procedure Localize; dynamic;
    procedure Suggest; dynamic;
    procedure SetModalResults;
  public
    { Public declarations }
    procedure Init(AChecker: TdxSpellChecker; const AMisspelledWord: String;
      AStyle: TFontInfo);
    function IsReplacingItemText: Boolean; dynamic;
    function GetStringsForSuggestions: TStrings; dynamic;
    function GetCorrectedText: String; dynamic;
  end;

implementation
uses cxClasses, dxSpellCheckerStrs, RVDXSpell;

{$R *.dfm}
{==================== TfrmRVDXSpellCheckerCustomForm ==========================}
procedure TfrmRVDXSpellCheckerCustomForm.FormCreate(Sender: TObject);
begin
  SetModalResults;
  Localize;
end;
{------------------------------------------------------------------------------}
function TfrmRVDXSpellCheckerCustomForm.GetCorrectedText: String;
begin
  Result := '';
end;
{------------------------------------------------------------------------------}
function TfrmRVDXSpellCheckerCustomForm.GetStringsForSuggestions: TStrings;
begin
  Result := nil;
end;
{------------------------------------------------------------------------------}
function TfrmRVDXSpellCheckerCustomForm.IsReplacingItemText: Boolean;
begin
  Result := False;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerCustomForm.Localize;
begin
  Caption := cxGetResourceString(@sdxSpellCheckerSpellingFormCaption);
  lbMisspelled.Caption := cxGetResourceString(@sdxSpellCheckerNotInDictionary);
  lbSuggestions.Caption := cxGetResourceString(@sdxSpellCheckerSuggestions);
  btnIgnore.Caption := cxGetResourceString(@sdxSpellCheckerIgnoreButton);
  btnIgnoreAll.Caption := cxGetResourceString(@sdxSpellCheckerIgnoreAllButton);
  btnChange.Caption := cxGetResourceString(@sdxSpellCheckerChangeButton);
  btnChangeAll.Caption := cxGetResourceString(@sdxSpellCheckerChangeAllButton);
  btnAdd.Caption := cxGetResourceString(@sdxSpellCheckerAddButton);
  btnOptions.Caption := cxGetResourceString(@sdxSpellCheckerOptionsButton);
  btnCancel.Caption := cxGetResourceString(@sdxSpellCheckerCancelButton);
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerCustomForm.SetModalResults;
begin
  btnIgnore.ModalResult := mrNo;
  btnIgnoreAll.ModalResult := mrNoToAll;
  btnChange.ModalResult := mrYes;
  btnChangeAll.ModalResult := mrYesToAll;
  btnAdd.ModalResult := mrOk;
  btnCancel.ModalResult := mrCancel;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerCustomForm.Suggest;
var sg: TStrings;
    Suggestions: TdxSpellCheckerSuggestionList;
begin
  Suggestions := FChecker.GetSuggestions(FMisspelledWord);
  FHasSuggestions := Suggestions.Count > 0;
  try
    sg := GetStringsForSuggestions;
    sg.BeginUpdate;
    try
      sg.Clear;
      if FHasSuggestions then
        Suggestions.SaveToStrings(sg)
      else
        sg.Add(cxGetResourceString(@sdxSpellCheckerNoSuggestions));
    finally
      sg.EndUpdate;
    end;
  finally
    Suggestions.Free;
  end;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerCustomForm.btnOptionsClick(Sender: TObject);
begin
  dxShowSpellingOptionsDialog(FChecker, FChecker.DialogLookAndFeel);
  TRVdxSpellChecker(FChecker).UpdateRules;
  Suggest;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerCustomForm.Init(AChecker: TdxSpellChecker;
  const AMisspelledWord: String; AStyle: TFontInfo);
begin
  FChecker := AChecker;
  SetControlLookAndFeel(Self, FChecker.DialogLookAndFeel);
  FMisspelledWord := AMisspelledWord;
  FTextStyle := AStyle;
  Suggest;
end;


end.
