
{*******************************************************}
{                                                       }
{       TfrmRVDXSpellCheckerOutlookForm -               }
{       MS Outlook-style form for  checking  spelling   }
{       in TRichViewEdit controls using                 }
{       ExpressSpellChecker                             }
{                                                       }
{       TRichView (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{       ExpressSpellChecker (c) Developer Express Inc.  }
{       http://www.devexpress.com                       }
{                                                       }
{*******************************************************}

unit RVDXSpellOutlookForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, dxCore,
  Dialogs, RVDXSpellCustomForm, Menus, cxLookAndFeelPainters, StdCtrls,
  cxButtons, cxEdit, cxTextEdit, cxControls, cxContainer, cxListBox,
  dxSpellChecker, RVTypes, RVStyle, cxGraphics, cxLookAndFeels;

type
  TfrmRVDXSpellCheckerOutlookForm = class(TfrmRVDXSpellCheckerCustomForm)
    lbxSuggestions: TcxListBox;
    teSuggestion: TcxTextEdit;
    teMisspelledWord: TcxTextEdit;
    lbChangeTo: TLabel;
    procedure teSuggestionPropertiesChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure lbxSuggestionsDblClick(Sender: TObject);
    procedure lbxSuggestionsClick(Sender: TObject);
  private
    { Private declarations }
    FSuggestionLockChange: Boolean;
    procedure SetSuggestionText(const AText: TRVUnicodeString);
    procedure ValidateCaptions;
  protected
    procedure Localize; override;
    procedure Suggest; override;
  public
    { Public declarations }
    function GetStringsForSuggestions: TStrings; override;
    function GetCorrectedText: String; override;
    procedure Init(AChecker: TdxSpellChecker; const AMisspelledWord: String;
      AStyle: TFontInfo);
  end;

implementation
uses cxClasses, dxSpellCheckerStrs, RVDXSpell;

{$R *.dfm}

{======================= TfrmRVDXSpellCheckerOutlookForm ======================}
procedure TfrmRVDXSpellCheckerOutlookForm.FormActivate(Sender: TObject);
begin
  teSuggestion.Text := '';
  teSuggestion.SetFocus;
  if FHasSuggestions then begin
    lbxSuggestions.ItemIndex := 0;
    SetSuggestionText(lbxSuggestions.Items[0]);
  end;
  ValidateCaptions;  
end;
{------------------------------------------------------------------------------}
function TfrmRVDXSpellCheckerOutlookForm.GetCorrectedText: String;
begin
  Result := teSuggestion.Text;
end;
{------------------------------------------------------------------------------}
function TfrmRVDXSpellCheckerOutlookForm.GetStringsForSuggestions: TStrings;
begin
  Result := lbxSuggestions.Items;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerOutlookForm.Init(AChecker: TdxSpellChecker;
  const AMisspelledWord: String; AStyle: TFontInfo);
begin
  inherited Init(AChecker, AMisspelledWord, AStyle);
  teMisspelledWord.Style.Font.Charset := AStyle.Charset;
  teSuggestion.Style.Font.Charset := AStyle.Charset;
  lbxSuggestions.Style.Font.Charset := AStyle.Charset;
  teMisspelledWord.Text := AMisspelledWord;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerOutlookForm.lbxSuggestionsClick(Sender: TObject);
begin
  if FHasSuggestions and (lbxSuggestions.ItemIndex>=0) then
    teSuggestion.Text := lbxSuggestions.Items[lbxSuggestions.ItemIndex];
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerOutlookForm.lbxSuggestionsDblClick(
  Sender: TObject);
begin
  if FHasSuggestions and (lbxSuggestions.ItemIndex>=0) then
    ModalResult := btnChange.ModalResult;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerOutlookForm.Localize;
begin
  inherited;
  lbChangeTo.Caption := cxGetResourceString(@sdxSpellCheckerChangeTo);
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerOutlookForm.SetSuggestionText(
  const AText: TRVUnicodeString);
begin
  FSuggestionLockChange := True;
  teSuggestion.Text := AText;
  teSuggestion.SelectAll;
  FSuggestionLockChange := False;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerOutlookForm.Suggest;
begin
  inherited Suggest;
  btnChange.Default := FHasSuggestions;
  btnIgnore.Default := not FHasSuggestions;
  lbxSuggestions.Enabled := FHasSuggestions;
  lbSuggestions.Enabled := FHasSuggestions;
  ValidateCaptions;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerOutlookForm.teSuggestionPropertiesChange(
  Sender: TObject);
begin
  if FSuggestionLockChange then
    Exit;
  ValidateCaptions
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerOutlookForm.ValidateCaptions;
begin
  if teSuggestion.Text<>'' then begin
    btnChange.Caption := cxGetResourceString(@sdxSpellCheckerChangeButton);
    btnChangeAll.Caption := cxGetResourceString(@sdxSpellCheckerChangeAllButton);
    end
  else begin
    btnChange.Caption := cxGetResourceString(@sdxSpellCheckerDeleteButton);
    btnChangeAll.Caption := cxGetResourceString(@sdxSpellCheckerDeleteAllButton);
  end;
end;

end.
