
{*******************************************************}
{                                                       }
{       TfrmRVDXSpellCheckerCustomForm - MS Word-style  }
{       form for  checking  spelling in TRichViewEdit   }
{       controls using  ExpressSpellChecker             }
{                                                       }
{       TRichView (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{       ExpressSpellChecker (c) Developer Express Inc.  }
{       http://www.devexpress.com                       }
{                                                       }
{*******************************************************}

unit RVDXSpellWordForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, RVDXSpellCustomForm, Menus, cxLookAndFeelPainters, StdCtrls,
  dxCore,
  cxButtons, cxListBox, cxControls, cxContainer, cxEdit, cxTextEdit, cxMemo,
  cxRichEdit, dxSpellChecker, RVStyle, cxGraphics, cxLookAndFeels;

type
  TfrmRVDXSpellCheckerWordForm = class(TfrmRVDXSpellCheckerCustomForm)
    btnUndoEdit: TcxButton;
    reMisspelledText: TcxRichEdit;
    lbxSuggestions: TcxListBox;
    procedure FormActivate(Sender: TObject);
    procedure lbxSuggestionsEnter(Sender: TObject);
    procedure lbxSuggestionsDblClick(Sender: TObject);
    procedure reMisspelledTextPropertiesChange(Sender: TObject);
    procedure btnUndoEditClick(Sender: TObject);
    procedure btnOptionsClick(Sender: TObject);
  private
    { Private declarations }
    FText: String;
    FStart, FLength: Integer;
    FReplacingItemText: Boolean;
    FUpdatingRichEdit: Boolean;
  protected
    procedure Localize; override;
    procedure FillRichEdit;
    procedure RemoveRed;
    procedure UpdateControlsStates;
    procedure Suggest; override;
  public
    { Public declarations }
    function IsReplacingItemText: Boolean; override;
    function GetStringsForSuggestions: TStrings; override;
    function GetCorrectedText: String; override;
    procedure Init(AChecker: TdxSpellChecker; const AMisspelledWord: String;
      const AText: String; AStart, ALength: Integer; AStyle: TFontInfo);
  end;

implementation
uses cxClasses, dxSpellCheckerStrs, RVDXSpell, RichEdit;

{$R *.dfm}

{======================== TfrmRVDXSpellCheckerWordForm ========================}
procedure TfrmRVDXSpellCheckerWordForm.FormActivate(Sender: TObject);
begin
  reMisspelledText.SetFocus;
  btnUndoEdit.Default := True;
  SendMessage(reMisspelledText.InnerControl.Handle, EM_SETUNDOLIMIT, 0, 0);
  if FHasSuggestions then
    lbxSuggestions.ItemIndex := 0;
  UpdateControlsStates;
end;
{------------------------------------------------------------------------------}
function TfrmRVDXSpellCheckerWordForm.GetCorrectedText: String;
begin
  if FReplacingItemText then
    Result := reMisspelledText.Text
  else if lbxSuggestions.ItemIndex>=0 then
    Result := lbxSuggestions.Items[lbxSuggestions.ItemIndex]
  else
    Result := '';
end;
{------------------------------------------------------------------------------}
function TfrmRVDXSpellCheckerWordForm.GetStringsForSuggestions: TStrings;
begin
  Result := lbxSuggestions.Items;
end;
{------------------------------------------------------------------------------}
function TfrmRVDXSpellCheckerWordForm.IsReplacingItemText: Boolean;
begin
  Result := FReplacingItemText;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerWordForm.lbxSuggestionsDblClick(Sender: TObject);
begin
  if FHasSuggestions and (lbxSuggestions.ItemIndex>=0) then
    ModalResult := btnChange.ModalResult;

end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerWordForm.lbxSuggestionsEnter(Sender: TObject);
begin
  btnIgnore.Default := False;
  btnChange.Default := True;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerWordForm.btnOptionsClick(Sender: TObject);
begin
  inherited;
  UpdateControlsStates;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerWordForm.btnUndoEditClick(Sender: TObject);
begin
  FReplacingItemText := False;
  FillRichEdit;
  reMisspelledText.SetFocus;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerWordForm.FillRichEdit;
begin
  FUpdatingRichEdit := True;
  try
    with reMisspelledText do begin
      Clear;
      SelAttributes.Charset := FTextStyle.Charset;
      SelAttributes.Name := FTextStyle.FontName;
      SelAttributes.Size := FTextStyle.Size;
      SelAttributes.Style := FTextStyle.Style;
      SelAttributes.Color := clDefault;
      SelText := FText;
      SendMessage(InnerControl.Handle, EM_HIDESELECTION, 1, 0);
      try
        SetSelection(FStart, FLength);
        SelAttributes.Color := clRed;
        SelAttributes.Style := SelAttributes.Style+[fsBold];
        SelStart := FStart+FLength;
      finally
        SendMessage(InnerControl.Handle, EM_HIDESELECTION, 0, 0);
      end;
    end;
    FReplacingItemText := False;
  finally
    FUpdatingRichEdit := False;
  end;
  UpdateControlsStates;
  btnChange.Default := False;
  btnIgnore.Default := True;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerWordForm.RemoveRed;
var LSelStart, LLength: Integer;
begin
  FUpdatingRichEdit := True;
  try
    with reMisspelledText do begin
      LSelStart := SelStart;
      LLength := SelLength;
      SendMessage(InnerControl.Handle, EM_HIDESELECTION, 1, 0);
      try
        SetSelection(0, Length(Text));
        SelAttributes.Charset := FTextStyle.Charset;
        SelAttributes.Name := FTextStyle.FontName;
        SelAttributes.Size := FTextStyle.Size;
        SelAttributes.Style := FTextStyle.Style;
        SelAttributes.Color := clDefault;
        SelStart := LSelStart;
        SelLength := LLength;
      finally
        SendMessage(InnerControl.Handle, EM_HIDESELECTION, 0, 0);
      end;
    end;
  finally
    FUpdatingRichEdit := False;
  end;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerWordForm.Init(AChecker: TdxSpellChecker;
  const AMisspelledWord: String; const AText: String;
  AStart, ALength: Integer; AStyle: TFontInfo);
begin
  inherited Init(AChecker, AMisspelledWord, AStyle);
  FText := AText;
  FStart := AStart;
  FLength := ALength;
  FillRichEdit;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerWordForm.Localize;
begin
  inherited;
  btnUndoEdit.Caption := cxGetResourceString(@sdxSpellCheckerUndoEditButton);
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerWordForm.reMisspelledTextPropertiesChange(
  Sender: TObject);
begin
  if FUpdatingRichEdit or FReplacingItemText then
    exit;
  FReplacingItemText := True;
  RemoveRed;
  UpdateControlsStates;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerWordForm.Suggest;
begin
  inherited Suggest;
  lbSuggestions.Enabled := FHasSuggestions;
  lbxSuggestions.Enabled := FHasSuggestions;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVDXSpellCheckerWordForm.UpdateControlsStates;
begin
  btnChangeAll.Enabled := not FReplacingItemText and FHasSuggestions;
  btnAdd.Enabled := not FReplacingItemText;
  lbSuggestions.Enabled := not FReplacingItemText and FHasSuggestions;
  lbxSuggestions.Enabled := not FReplacingItemText and FHasSuggestions;
  btnChange.Enabled := FReplacingItemText or FHasSuggestions;
  btnUndoEdit.Visible := FReplacingItemText;
  if FReplacingItemText then begin
    btnUndoEdit.Default := btnIgnore.Default;
    btnIgnore.Default := False;
    end
  else begin
    btnIgnore.Default := btnUndoEdit.Default;
    btnUndoEdit.Default := False;
  end;
end;

end.
