
{*******************************************************}
{                                                       }
{       RichView                                        }
{       TRVSpellDlg: component for                      }
{       spell-checking RichViewEdit using EDSSpell 7.0  }
{       by OneDomain, Inc                               }
{                                                       }
{       RichView: Copyright (c) Sergey Tkachenko        }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{       EDSSpell : Copyright (c) OneDomain, Inc         }
{       http://www.onedomain.com                        }
{                                                       }
{*******************************************************}

{

  Installing this component:
    - Add this file (RVEDSSpell) in some package. Compile this package.
      TRVSpellDlg component will appear on "RichView" page of the
      Component Palette. This component should be used instead of TSpellDlg.

  Using:
    - place TRVSpellDlg on the form.
    - in program, call TRVSpellDlg.Check(rve, rvesFromCursor) to check
      rve: TRichViewEdit from the cursor position.
      Call TRVSpellDlg.Check(rve, rvesFromStart) to check from the
      beginning.
  Functions for live spelling:
    Important note: call TRVSpellDlg.OpenDictionary before starting
    a live spelling check (for example, in Form.OnCreate)
    - checking word in RichViewEdit.OnSpellingCheck: TRVSpellDlg.IsBadWord
    - adding to ignore list: TRVSpellDlg.SkipList.Add
    - adding to dictionary: TRVSpellDlg.AddWord
    - suggestions: SuggestWords
    - event on adding word to ignore list: OnSpellFormAction(..., mrSkipAll)
    - event on adding word to dictionary: OnSpellFormAction(..., mrAdd)

  History:
  2011-03-20: property DialogShown specifies if a spelling check dialog has been
    shown during the last spelling check.
  2009-02-11: OnSpellFormAction
  2004-07-04: comments about live spelling
}

unit RVEDSSpell;

interface

uses SysUtils, Classes, Controls, Forms,
     RVEdit,
     RVWordEnum, RVTypes,
     EDSSpell, SpellFrm;

type
  TRVSpellDlg = class;

  {
   Possible values for Action parameter:
   - mrReplace
   - mrAdd
   - mrSkipOnce
   - mrSkipAll
   - mrReplaceAll
   (defined in SpellFrm unit)
  }

  TRVEDSSpellFormActionEvent = procedure(Sender: TRVSpellDlg;
    const AWord, AReplaceTo: String; Action: Integer) of object;

  TRVSpellDlg = class (TCustomSpellDlg)
  private
    FOnSpellFormAction: TRVEDSSpellFormActionEvent;
    FDialogShown: Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Check(Editor: TCustomRichViewEdit; Scope: TRVEnumScope);
    property DialogShown: Boolean read FDialogShown;
  protected
    property Replacer;
    property ReplaceList;
    property SkipList;
  published
    property AccentedCharacters;
    property AutoCorrectList;
    property AutoShift;
    property AutoSuggest;
    property ConfirmAdd;
    property DictionaryName;
    property DictionaryPath;
    property ExclusionList;
    property Font;
    property HelpContext;
    property IgnoreAllUpperCase;
    property IgnoreWithNumbers;
    property LabelLanguage;
    property SpellCheckMode;
    property Suggestions;
    property ShowCompletedDialog;
    property ShowGlyphs;
    property SupportHTML;
    property SupportRTF2;
    property UserDictionary;

    property OnCreateSpellForm;
    property OnAddWordMsg;
    property OnWordAlreadyAddedMsg;
    property OnSpellFormAction: TRVEDSSpellFormActionEvent read
      FOnSpellFormAction write FOnSpellFormAction;
  end;


  TRVEDSSpell7Helper = class (TRVWordEnumerator)
    private
      FSpellDlg: TRVSpellDlg;
    protected
      function ProcessWord: Boolean; override;
    public
      procedure Check(Editor: TCustomRichViewEdit; Scope: TRVEnumScope);
    published
      property SpellDlg: TRVSpellDlg read FSpellDlg write FSpellDlg;
  end;


procedure Register;

implementation

{========================== TRVEDSSpell7Helper ================================}
procedure TRVEDSSpell7Helper.Check(Editor: TCustomRichViewEdit;
  Scope: TRVEnumScope);

begin
  if FSpellDlg=nil then
    raise Exception.Create('SpellDlg is not assigned');
  Run(Editor, Scope);
end;
{------------------------------------------------------------------------------}
function TRVEDSSpell7Helper.ProcessWord: Boolean;
var OldWord : TRVAnsiString;
    r: Integer;
begin
  r := FSpellDlg.ReplaceList.IndexOf(FWordA);
  if r>=0 then begin
    FWordA := FSpellDlg.Replacer[r];
    FWordChanged := True;
    Result := True;
    exit;
  end;
  Result := not FSpellDlg.IsBadWord(FWordA);
  if Result then
    exit;
  SelectCurrentWord;
  OldWord := FWordA;
  Result := True;
  FDialogShown := True;
  r := FSpellDlg.CheckWord(String(FWordA));
  if (r<>mrCancel) and Assigned(FSpellDlg.OnSpellFormAction) then
    FSpellDlg.OnSpellFormAction(FSpellDlg, OldWord, FWordA, r); 
  case r of
    mrReplace:
      FWordChanged := FWordA<>OldWord;
    mrReplaceAll:
      begin
        FWordChanged := FWordA<>OldWord;
        if FWordChanged then begin
          FSpellDlg.ReplaceList.Add(AnsiLowerCase(OldWord));
          FSpellDlg.Replacer.Add(FWordA);
        end;
      end;
    mrAdd:
      begin
        FSpellDlg.AddWord(AnsiLowerCase(OldWord));
        // FDictionary.WriteAddedList;
      end;
    mrSkipOnce:
      ;
    mrSkipAll:
      FSpellDlg.SkipList.Add(AnsiLowerCase(OldWord));
    mrCancel:
      Result := False;
  end;
  FSpellDlg.CloseSpellWin;
end;
{=============================== TRVSpellDlg ==================================}
procedure TRVSpellDlg.Check(Editor: TCustomRichViewEdit;
  Scope: TRVEnumScope);
var Helper: TRVEDSSpell7Helper;
begin
  Helper := TRVEDSSpell7Helper.Create;
  try
    Helper.SpellDlg := Self;
    Helper.Check(Editor, Scope);
    FDialogShown := Helper.DialogShown;
  finally
    Helper.Free;
  end;
end;
{------------------------------------------------------------------------------}
constructor TRVSpellDlg.Create(AOwner: TComponent);
begin
  inherited;
end;
{------------------------------------------------------------------------------}
destructor TRVSpellDlg.Destroy;
begin
  inherited;
end;
{==============================================================================}
procedure Register;
begin
  RegisterComponents('RichView', [TRVSpellDlg]);
end;

end.