
{*******************************************************}
{                                                       }
{       RichView                                        }
{       Conversion ISO 8859 <--> Unicode                }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************}

{
  The following ISO 8859 versions are supported: 1..10, 13..15.
}


unit RVISO8859;

interface
uses Windows, Classes,
  RVStyle, RVTypes, RVUni, RVClasses;

// Return value: True if lossless.
// ISO -> Unicode: False if the original string is not correct (does not contain
//  undefined characters), undefined characters are converted to '?'
// Unicode -> : False if the conversion loses some characters (converted to
//  '?').
//  Besides, False is returned if IsoVersion is incorrect/unsupported
function ConvertISO8859AndUnicode(var iso: TRVRawByteString; var uni: TRVUnicodeString;
  IsoVersion: Integer; ToUnicode: Boolean): Boolean;
// Allocates memory required to convert from Unicode to ISO.
// This memory is automatically allocated by ConvertISO8859AndUnicode,
// but since it can be called in thread context, it is desirable to allocate
// it before (since this memory will be freed in the context of the main
// process
procedure AllocateISO8859Table(IsoVersion: Integer);

implementation
type
  TIsoToUnicodeTable = array[160..255] of Word;
  PIsoToUnicodeTable = ^TIsoToUnicodeTable;
  TPair = class
    public
      uni: Word;
      iso: Byte;
  end;
  TPairList = class (TRVList)
    public
      function Find(uni: Word): Byte;
  end;
  TPairListArray = array [2..15] of TPairList;
  TTableArray = array[2..15] of PIsoToUnicodeTable;

const Table2: TIsoToUnicodeTable =
 ($00A0, $0104, $02D8, $0141, $00A4, $013D, $015A, $00A7,
  $00A8, $0160, $015E, $0164, $0179, $00AD, $017D, $017B,
  $00B0, $0105, $02DB, $0142, $00B4, $013E, $015B, $02C7,
  $00B8, $0161, $015F, $0165, $017A, $02DD, $017E, $017C,
  $0154, $00C1, $00C2, $0102, $00C4, $0139, $0106, $00C7,
  $010C, $00C9, $0118, $00CB, $011A, $00CD, $00CE, $010E,
  $0110, $0143, $0147, $00D3, $00D4, $0150, $00D6, $00D7,
  $0158, $016E, $00DA, $0170, $00DC, $00DD, $0162, $00DF,
  $0155, $00E1, $00E2, $0103, $00E4, $013A, $0107, $00E7,
  $010D, $00E9, $0119, $00EB, $011B, $00ED, $00EE, $010F,
  $0111, $0144, $0148, $00F3, $00F4, $0151, $00F6, $00F7,
  $0159, $016F, $00FA, $0171, $00FC, $00FD, $0163, $02D9);
Table3: TIsoToUnicodeTable =
 ($00A0, $0126, $02D8, $00A3, $00A4, $003F, $0124, $00A7,
  $00A8, $0130, $015E, $011E, $0134, $00AD, $003F, $017B,
  $00B0, $0127, $00B2, $00B3, $00B4, $00B5, $0125, $00B7,
  $00B8, $0131, $015F, $011F, $0135, $00BD, $003F, $017C,
  $00C0, $00C1, $00C2, $003F, $00C4, $010A, $0108, $00C7,
  $00C8, $00C9, $00CA, $00CB, $00CC, $00CD, $00CE, $00CF,
  $003F, $00D1, $00D2, $00D3, $00D4, $0120, $00D6, $00D7,
  $011C, $00D9, $00DA, $00DB, $00DC, $016C, $015C, $00DF,
  $00E0, $00E1, $00E2, $003F, $00E4, $010B, $0109, $00E7,
  $00E8, $00E9, $00EA, $00EB, $00EC, $00ED, $00EE, $00EF,
  $003F, $00F1, $00F2, $00F3, $00F4, $0121, $00F6, $00F7,
  $011D, $00F9, $00FA, $00FB, $00FC, $016D, $015D, $02D9);
Table4: TIsoToUnicodeTable =
 ($00A0, $0104, $0138, $0156, $00A4, $0128, $013B, $00A7,
  $00A8, $0160, $0112, $0122, $0166, $00AD, $017D, $00AF,
  $00B0, $0105, $02DB, $0157, $00B4, $0129, $013C, $02C7,
  $00B8, $0161, $0113, $0123, $0167, $014A, $017E, $014B,
  $0100, $00C1, $00C2, $00C3, $00C4, $00C5, $00C6, $012E,
  $010C, $00C9, $0118, $00CB, $0116, $00CD, $00CE, $012A,
  $0110, $0145, $014C, $0136, $00D4, $00D5, $00D6, $00D7,
  $00D8, $0172, $00DA, $00DB, $00DC, $0168, $016A, $00DF,
  $0101, $00E1, $00E2, $00E3, $00E4, $00E5, $00E6, $012F,
  $010D, $00E9, $0119, $00EB, $0117, $00ED, $00EE, $012B,
  $0111, $0146, $014D, $0137, $00F4, $00F5, $00F6, $00F7,
  $00F8, $0173, $00FA, $00FB, $00FC, $0169, $016B, $02D9);
Table5: TIsoToUnicodeTable =
 ($00A0, $0401, $0402, $0403, $0404, $0405, $0406, $0407,
  $0408, $0409, $040A, $040B, $040C, $00AD, $040E, $040F,
  $0410, $0411, $0412, $0413, $0414, $0415, $0416, $0417,
  $0418, $0419, $041A, $041B, $041C, $041D, $041E, $041F,
  $0420, $0421, $0422, $0423, $0424, $0425, $0426, $0427,
  $0428, $0429, $042A, $042B, $042C, $042D, $042E, $042F,
  $0430, $0431, $0432, $0433, $0434, $0435, $0436, $0437,
  $0438, $0439, $043A, $043B, $043C, $043D, $043E, $043F,
  $0440, $0441, $0442, $0443, $0444, $0445, $0446, $0447,
  $0448, $0449, $044A, $044B, $044C, $044D, $044E, $044F,
  $2116, $0451, $0452, $0453, $0454, $0455, $0456, $0457,
  $0458, $0459, $045A, $045B, $045C, $00A7, $045E, $045F);
Table6: TIsoToUnicodeTable =
 ($00A0, $0000, $0000, $0000, $00A4, $0000, $0000, $0000,
  $0000, $0000, $0000, $0000, $060C, $00AD, $0000, $0000,
  $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000,
  $0000, $0000, $0000, $061B, $0000, $0000, $0000, $061F,
  $0000, $0621, $0622, $0623, $0624, $0625, $0626, $0627,
  $0628, $0629, $062A, $062B, $062C, $062D, $062E, $062F,
  $0630, $0631, $0632, $0633, $0634, $0635, $0636, $0637,
  $0638, $0639, $063A, $0000, $0000, $0000, $0000, $0000,
  $0640, $0641, $0642, $0643, $0644, $0645, $0646, $0647,
  $0648, $0649, $064A, $064B, $064C, $064D, $064E, $064F,
  $0650, $0651, $0652, $0000, $0000, $0000, $0000, $0000,
  $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000);
Table7: TIsoToUnicodeTable =
 ($00A0, $2018, $2019, $00A3, $20AC, $20AF, $00A6, $00A7,
  $00A8, $00A9, $037A, $00AB, $00AC, $00AD, $0000, $2015,
  $00B0, $00B1, $00B2, $00B3, $0384, $0385, $0386, $00B7,
  $0388, $0389, $038A, $00BB, $038C, $00BD, $038E, $038F,
  $0390, $0391, $0392, $0393, $0394, $0395, $0396, $0397,
  $0398, $0399, $039A, $039B, $039C, $039D, $039E, $039F,
  $03A0, $03A1, $0000, $03A3, $03A4, $03A5, $03A6, $03A7,
  $03A8, $03A9, $03AA, $03AB, $03AC, $03AD, $03AE, $03AF,
  $03B0, $03B1, $03B2, $03B3, $03B4, $03B5, $03B6, $03B7,
  $03B8, $03B9, $03BA, $03BB, $03BC, $03BD, $03BE, $03BF,
  $03C0, $03C1, $03C2, $03C3, $03C4, $03C5, $03C6, $03C7,
  $03C8, $03C9, $03CA, $03CB, $03CC, $03CD, $03CE, $0000);
Table8: TIsoToUnicodeTable =
 ($00A0, $0000, $00A2, $00A3, $00A4, $00A5, $00A6, $00A7,
  $00A8, $00A9, $00D7, $00AB, $00AC, $00AD, $00AE, $00AF,
  $00B0, $00B1, $00B2, $00B3, $00B4, $00B5, $00B6, $00B7,
  $00B8, $00B9, $00F7, $00BB, $00BC, $00BD, $00BE, $0000,
  $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000,
  $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000,
  $0000, $0000, $0000, $0000, $0000, $0000, $0000, $0000,
  $0000, $0000, $0000, $0000, $0000, $0000, $0000, $2017,
  $05D0, $05D1, $05D2, $05D3, $05D4, $05D5, $05D6, $05D7,
  $05D8, $05D9, $05DA, $05DB, $05DC, $05DD, $05DE, $05DF,
  $05E0, $05E1, $05E2, $05E3, $05E4, $05E5, $05E6, $05E7,
  $05E8, $05E9, $05EA, $0000, $0000, $200E, $200F, $0000);
Table9: TIsoToUnicodeTable =
 ($00A0, $00A1, $00A2, $00A3, $00A4, $00A5, $00A6, $00A7,
  $00A8, $00A9, $00AA, $00AB, $00AC, $00AD, $00AE, $00AF,
  $00B0, $00B1, $00B2, $00B3, $00B4, $00B5, $00B6, $00B7,
  $00B8, $00B9, $00BA, $00BB, $00BC, $00BD, $00BE, $00BF,
  $00C0, $00C1, $00C2, $00C3, $00C4, $00C5, $00C6, $00C7,
  $00C8, $00C9, $00CA, $00CB, $00CC, $00CD, $00CE, $00CF,
  $011E, $00D1, $00D2, $00D3, $00D4, $00D5, $00D6, $00D7,
  $00D8, $00D9, $00DA, $00DB, $00DC, $0130, $015E, $00DF,
  $00E0, $00E1, $00E2, $00E3, $00E4, $00E5, $00E6, $00E7,
  $00E8, $00E9, $00EA, $00EB, $00EC, $00ED, $00EE, $00EF,
  $011F, $00F1, $00F2, $00F3, $00F4, $00F5, $00F6, $00F7,
  $00F8, $00F9, $00FA, $00FB, $00FC, $0131, $015F, $00FF);
Table10: TIsoToUnicodeTable =
 ($00A0, $0104, $0112, $0122, $012A, $0128, $0136, $00A7,
  $013B, $0110, $0160, $0166, $017D, $00AD, $016A, $014A,
  $00B0, $0105, $0113, $0123, $012B, $0129, $0137, $00B7,
  $013C, $0111, $0161, $0167, $017E, $2015, $016B, $014B,
  $0100, $00C1, $00C2, $00C3, $00C4, $00C5, $00C6, $012E,
  $010C, $00C9, $0118, $00CB, $0116, $00CD, $00CE, $00CF,
  $00D0, $0145, $014C, $00D3, $00D4, $00D5, $00D6, $0168,
  $00D8, $0172, $00DA, $00DB, $00DC, $00DD, $00DE, $00DF,
  $0101, $00E1, $00E2, $00E3, $00E4, $00E5, $00E6, $012F,
  $010D, $00E9, $0119, $00EB, $0117, $00ED, $00EE, $00EF,
  $00F0, $0146, $014D, $00F3, $00F4, $00F5, $00F6, $0169,
  $00F8, $0173, $00FA, $00FB, $00FC, $00FD, $00FE, $0138);
Table13: TIsoToUnicodeTable =
 ($00A0, $201D, $00A2, $00A3, $00A4, $201E, $00A6, $00A7,
  $00D8, $00A9, $0156, $00AB, $00AC, $00AD, $00AE, $00C6,
  $00B0, $00B1, $00B2, $00B3, $201C, $00B5, $00B6, $00B7,
  $00F8, $00B9, $0157, $00BB, $00BC, $00BD, $00BE, $00E6,
  $0104, $012E, $0100, $0106, $00C4, $00C5, $0118, $0112,
  $010C, $00C9, $0179, $0116, $0122, $0136, $012A, $013B,
  $0160, $0143, $0145, $00D3, $014C, $00D5, $00D6, $00D7,
  $0172, $0141, $015A, $016A, $00DC, $017B, $017D, $00DF,
  $0105, $012F, $0101, $0107, $00E4, $00E5, $0119, $0113,
  $010D, $00E9, $017A, $0117, $0123, $0137, $012B, $013C,
  $0161, $0144, $0146, $00F3, $014D, $00F5, $00F6, $00F7,
  $0173, $0142, $015B, $016B, $00FC, $017C, $017E, $2019);
Table14: TIsoToUnicodeTable =
 ($00A0, $1E02, $1E03, $00A3, $010A, $010B, $1E0A, $00A7,
  $1E80, $00A9, $1E82, $1E0B, $1EF2, $00AD, $00AE, $0178,
  $1E1E, $1E1F, $0120, $0121, $1E40, $1E41, $00B6, $1E56,
  $1E81, $1E57, $1E83, $1E60, $1EF3, $1E84, $1E85, $1E61,
  $00C0, $00C1, $00C2, $00C3, $00C4, $00C5, $00C6, $00C7,
  $00C8, $00C9, $00CA, $00CB, $00CC, $00CD, $00CE, $00CF,
  $0174, $00D1, $00D2, $00D3, $00D4, $00D5, $00D6, $1E6A,
  $00D8, $00D9, $00DA, $00DB, $00DC, $00DD, $0176, $00DF,
  $00E0, $00E1, $00E2, $00E3, $00E4, $00E5, $00E6, $00E7,
  $00E8, $00E9, $00EA, $00EB, $00EC, $00ED, $00EE, $00EF,
  $0175, $00F1, $00F2, $00F3, $00F4, $00F5, $00F6, $1E6B,
  $00F8, $00F9, $00FA, $00FB, $00FC, $00FD, $0177, $00FF);
 Table15: TIsoToUnicodeTable =
 ($00A0, $00A1, $00A2, $00A3, $20AC, $00A5, $0160, $00A7,
  $0161, $00A9, $00AA, $00AB, $00AC, $00AD, $00AE, $00AF,
  $00B0, $00B1, $00B2, $00B3, $017D, $00B5, $00B6, $00B7,
  $017E, $00B9, $00BA, $00BB, $0152, $0153, $0178, $00BF,
  $00C0, $00C1, $00C2, $00C3, $00C4, $00C5, $00C6, $00C7,
  $00C8, $00C9, $00CA, $00CB, $00CC, $00CD, $00CE, $00CF,
  $00D0, $00D1, $00D2, $00D3, $00D4, $00D5, $00D6, $00D7,
  $00D8, $00D9, $00DA, $00DB, $00DC, $00DD, $00DE, $00DF,
  $00E0, $00E1, $00E2, $00E3, $00E4, $00E5, $00E6, $00E7,
  $00E8, $00E9, $00EA, $00EB, $00EC, $00ED, $00EE, $00EF,
  $00F0, $00F1, $00F2, $00F3, $00F4, $00F5, $00F6, $00F7,
  $00F8, $00F9, $00FA, $00FB, $00FC, $00FD, $00FE, $00FF);
var
 PairListArray: TPairListArray;
 TableArray: TTableArray;
{------------------------------------------------------------------------------}
function ComparePairs(Item1, Item2: Pointer): Integer;
begin
  if TPair(Item1).uni>TPair(Item2).uni then
    Result := 1
  else if TPair(Item1).uni<TPair(Item2).uni then
    Result := -1
  else
    Result := 0;
end;
function ComparePair(Item1: Pointer; uni: Word): Integer;
begin
  if TPair(Item1).uni>uni then
    Result := 1
  else if TPair(Item1).uni<uni then
    Result := -1
  else
    Result := 0;
end;
{------------------------------------------------------------------------------}
function TPairList.Find(uni: Word): Byte;
var
  L, H, I, C: Integer;
begin
  Result := 0;
  L := 0;
  H := Count - 1;
  while L <= H do begin
    I := (L + H) shr 1;
    C := ComparePair(Items[I], uni);
    if C < 0 then
      L := I + 1
    else begin
      H := I - 1;
      if C = 0 then begin
        Result := TPair(Items[I]).iso;
        break;
      end;
    end;
  end;
end;
{------------------------------------------------------------------------------}
function CreatePairArrayFromTable(PIsoTable: PIsoToUnicodeTable): TPairList;
var i, Count: Integer;
    Pair: TPair;
begin
  Count := 0;
  for i := 160 to 255 do
    if PIsoTable[i]<>0 then
      inc(Count);
  Result := TPairList.Create;
  Result.Capacity := Count;
  for i := 160 to 255 do
    if PIsoTable[i]<>0 then begin
      Pair := TPair.Create;
      Pair.uni := PIsoTable[i];
      Pair.iso := i;
      Result.Add(Pair);
    end;
  Result.Sort(@ComparePairs);
end;
{------------------------------------------------------------------------------}
function ConvertISO8859AndUnicode(var iso: TRVRawByteString; var uni: TRVUnicodeString;
  IsoVersion: Integer; ToUnicode: Boolean): Boolean;

var i: Integer;
  PTable: PIsoToUnicodeTable;
  PairList: TPairList;
begin
  if (IsoVersion>15) or (IsoVersion<1) then begin
    Result := False;
    exit;
  end;
  Result := True;
  if IsoVersion=1 then begin
    // Version 1 has the same codes as Unicode
    if ToUnicode then begin
      SetLength(uni, Length(iso));
      for i := 1 to Length(iso) do
        uni[i] := TRVUnicodeChar(ord(iso[i]));
      end
    else begin
      SetLength(iso, Length(uni));
      Result := True;
      for i := 1 to Length(uni) do
        if ord(uni[i])<256 then
          iso[i] := TRVAnsiChar(ord(uni[i]))
        else begin
          iso[i] := '?';
          Result := False;
        end;
    end;
    exit;
  end;
  PTable := TableArray[IsoVersion];
  if PTable=nil then begin
    Result := False;
    exit;
  end;
  if ToUnicode then begin
    SetLength(uni, Length(iso));
    for i := 1 to Length(iso) do
      if iso[i]<#160 then
        uni[i] := TRVUnicodeChar(ord(iso[i]))
      else begin
        uni[i] := TRVUnicodeChar(PTable[ord(iso[i])]);
        if ord(uni[i])=0 then begin
          uni[i] := '?';
          Result := False;
        end;
      end;
    end
  else begin
    if PairListArray[IsoVersion]=nil then
      PairListArray[IsoVersion] := CreatePairArrayFromTable(PTable);
    PairList := PairListArray[IsoVersion];
    SetLength(iso, Length(uni));
    for i := 1 to Length(uni) do
      if ord(uni[i])<160 then
        iso[i] := TRVAnsiChar(ord(uni[i]))
      else begin
        iso[i] := TRVAnsiChar(PairList.Find(ord(uni[i])));
        if ord(iso[i])=0 then begin
          iso[i] := '?';
          Result := False;
        end;
      end;
  end;
end;
{------------------------------------------------------------------------------}
procedure AllocateISO8859Table(IsoVersion: Integer);
begin
  if (IsoVersion>15) or (IsoVersion<1) then
    exit;
  if TableArray[IsoVersion]=nil then
    exit;
  if PairListArray[IsoVersion]=nil then
    PairListArray[IsoVersion] := CreatePairArrayFromTable(TableArray[IsoVersion]);
end;
{==============================================================================}
{$O-}
procedure InitArrays;
var i: Integer;
begin
  for i := 2 to 15 do
    PairListArray[i] := nil;
  TableArray[2] := @Table2;
  TableArray[3] := @Table3;
  TableArray[4] := @Table4;
  TableArray[5] := @Table5;
  TableArray[6] := @Table6;
  TableArray[7] := @Table7;
  TableArray[8] := @Table8;
  TableArray[9] := @Table9;
  TableArray[10] := @Table10;
  TableArray[11] := nil;
  TableArray[12] := nil;
  TableArray[13] := @Table13;
  TableArray[14] := @Table14;
  TableArray[15] := @Table15;
end;

procedure DoneArrays;
var i: Integer;
    PairList: TPairList;
begin
  for i := 2 to 15 do begin
    PairList := PairListArray[i];
    PairList.Free;
    PairListArray[i] := nil;
  end;
end;

initialization
  InitArrays;
finalization
  DoneArrays;  

end.