{*******************************************************}
{                                                       }
{       RichView                                        }
{       Form for spell checking in MS Word style        }
{                                                       }
{       RichView: Copyright (c) Sergey Tkachenko        }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************}

unit RVSpellWordFrm;

interface

{$I RV_Defs.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, RVCustomSpellFrm, StdCtrls, RVStyle, RVScroll, RichView, RVEdit,
  RVLinear,
  {$IFDEF RVUNICODESTR}
  RVGetTextW,
  {$ELSE}
  RVGetText,
  {$ENDIF}
  Clipbrd;

type
  TfrmRVSpellWord = class(TfrmRVCustomSpell)
    rve: TRichViewEdit;
    RVStyle1: TRVStyle;
    lst: TListBox;
    btnUndoEdit: TButton;
    procedure rveChange(Sender: TObject);
    procedure btnUndoEditClick(Sender: TObject);
    procedure rvePaste(Sender: TCustomRichViewEdit;
      var DoDefault: Boolean);
    procedure FormActivate(Sender: TObject);
    procedure lstEnter(Sender: TObject);
    procedure lstDblClick(Sender: TObject);
  private
    { Private declarations }
    FText: String;
    FStart, FLength: Integer;
    FReplacingItemText: Boolean;
    procedure FillRVEdit;
    procedure UpdateControlsStates;
  public
    { Public declarations }
    procedure Init(const AText: String; AStart, ALength: Integer;
      Style: TFontInfo);
    function GetCorrectedText: String; override;
    function IsReplacingItemText: Boolean; override;
    function GetStringsForSuggestions: TStrings; override;
  end;

implementation

{$R *.dfm}

procedure TfrmRVSpellWord.rveChange(Sender: TObject);
var SelStart, SelLength: Integer;
    s: String;
begin
  if not FReplacingItemText then begin
    RVGetSelection(rve, SelStart, SelLength);
    s := GetAllText(rve);
    rve.Clear;
    rve.AddNL(s, 0, 0);
    rve.Format;
    RVSetSelection(rve, SelStart, SelLength);
    FReplacingItemText := True;
    UpdateControlsStates;
  end;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVSpellWord.UpdateControlsStates;
begin
  btnChangeAll.Enabled := not FReplacingItemText;
  btnAdd.Enabled := not FReplacingItemText;
  lblCmb.Enabled := not FReplacingItemText;
  lst.Enabled := not FReplacingItemText;
  btnUndoEdit.Visible := FReplacingItemText;
  if FReplacingItemText then begin
    btnUndoEdit.Default := btnIgnore.Default;
    btnIgnore.Default := False;
    end
  else begin
    btnIgnore.Default := btnUndoEdit.Default;
    btnUndoEdit.Default := False;
  end;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVSpellWord.btnUndoEditClick(Sender: TObject);
begin
  FillRVEdit;
  rve.SetFocus;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVSpellWord.Init(const AText: String; AStart,
  ALength: Integer; Style: TFontInfo);
begin
  FText := AText;
  FStart := AStart;
  FLength := ALength;
  RVStyle1.TextStyles[0] := Style;
  RVStyle1.TextStyles[1] := Style;
  RVStyle1.TextStyles[0].Color := clWindowText;
  RVStyle1.TextStyles[1].Color := clRed;
  RVStyle1.TextStyles[1].Style := RVStyle1.TextStyles[1].Style + [fsBold];
  FillRVEdit;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVSpellWord.rvePaste(Sender: TCustomRichViewEdit;
  var DoDefault: Boolean);
var s: String;
begin
  s := Clipboard.AsText;
  if (Pos(#13,s)=0) and (Pos(#10,s)=0) then
    Sender.InsertText(s, False);
  DoDefault := False;
end;
{------------------------------------------------------------------------------}
function TfrmRVSpellWord.GetCorrectedText: String;
begin
  if FReplacingItemText then
    Result := GetAllText(rve)
  else if lst.ItemIndex>=0 then
    Result := lst.Items[lst.ItemIndex]
  else
    Result := '';
end;
{------------------------------------------------------------------------------}
procedure TfrmRVSpellWord.FillRVEdit;
var ErrorItemNo: Integer;
begin
  rve.Clear;
  ErrorItemNo := 0;
  if FStart>1 then begin
    rve.AddNL(Copy(FText, 1, FStart-1), 0, 0);
    inc(ErrorItemNo);
  end;
  rve.Add(Copy(FText, FStart, FLength), 1);
  rve.Add(Copy(FText, FStart+FLength, Length(FText)), 0);
  rve.Format;
  rve.SetSelectionBounds(ErrorItemNo, FLength+1, ErrorItemNo, FLength+1);
  FReplacingItemText := False;
  UpdateControlsStates;
  btnChange.Default := False;  
  btnIgnore.Default := True;
end;
{------------------------------------------------------------------------------}
function TfrmRVSpellWord.IsReplacingItemText: Boolean;
begin
  Result := FReplacingItemText;
end;
{------------------------------------------------------------------------------}
function TfrmRVSpellWord.GetStringsForSuggestions: TStrings;
begin
  Result := lst.Items;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVSpellWord.FormActivate(Sender: TObject);
begin
  rve.SetFocus;
  btnUndoEdit.Default := True;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVSpellWord.lstEnter(Sender: TObject);
begin
  btnIgnore.Default := False;
  btnChange.Default := True;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVSpellWord.lstDblClick(Sender: TObject);
begin
  ModalResult := btnChange.ModalResult;
end;

end.
