{*******************************************************}
{                                                       }
{       TRichView                                       }
{       Form for spell checking in MS Word style        }
{       (TNT-controls version)                          }
{                                                       }
{       RichView: Copyright (c) Sergey Tkachenko        }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************}

unit RVTntSpellWordFrm;

interface

{$I RV_Defs.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, RVStyle, RVScroll, RichView, RVEdit, RVTypes,
  RVLinear, RVGetTextW, RVUni,
  Clipbrd, RVTntCustomSpellFrm, TntStdCtrls, TntClasses;

type
  TfrmRVTntSpellWord = class(TfrmRVTntCustomSpell)
    rve: TRichViewEdit;
    RVStyle1: TRVStyle;
    lst: TTntListBox;
    btnUndoEdit: TTntButton;
    procedure rveChange(Sender: TObject);
    procedure btnUndoEditClick(Sender: TObject);
    procedure rvePaste(Sender: TCustomRichViewEdit;
      var DoDefault: Boolean);
    procedure FormActivate(Sender: TObject);
    procedure lstEnter(Sender: TObject);
    procedure lstDblClick(Sender: TObject);
  private
    { Private declarations }
    FText: WideString;
    FStart, FLength: Integer;
    FReplacingItemText: Boolean;
    procedure FillRVEdit;
    procedure UpdateControlsStates;
  public
    { Public declarations }
    procedure Init(const AText: WideString; AStart, ALength: Integer;
      Style: TFontInfo);
    function GetCorrectedText: WideString; override;
    function IsReplacingItemText: Boolean; override;
    function GetStringsForSuggestions: TTntStrings; override;
  end;

implementation

{$R *.dfm}

procedure TfrmRVTntSpellWord.rveChange(Sender: TObject);
var SelStart, SelLength: Integer;
    s: WideString;
begin
  if not FReplacingItemText then begin
    RVGetSelection(rve, SelStart, SelLength);
    s := GetAllText(rve);
    rve.Clear;
    rve.AddNLWTag(s, 0, 0, RVEMPTYTAG);
    rve.Format;
    RVSetSelection(rve, SelStart, SelLength);
    FReplacingItemText := True;
    UpdateControlsStates;
  end;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVTntSpellWord.UpdateControlsStates;
begin
  btnChangeAll.Enabled := not FReplacingItemText;
  btnAdd.Enabled := not FReplacingItemText;
  lblCmb.Enabled := not FReplacingItemText;
  lst.Enabled := not FReplacingItemText;
  btnUndoEdit.Visible := FReplacingItemText;
  if FReplacingItemText then begin
    btnUndoEdit.Default := btnIgnore.Default;
    btnIgnore.Default := False;
    end
  else begin
    btnIgnore.Default := btnUndoEdit.Default;
    btnUndoEdit.Default := False;
  end;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVTntSpellWord.btnUndoEditClick(Sender: TObject);
begin
  FillRVEdit;
  rve.SetFocus;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVTntSpellWord.Init(const AText: WideString; AStart,
  ALength: Integer; Style: TFontInfo);
begin
  FText := AText;
  FStart := AStart;
  FLength := ALength;
  RVStyle1.TextStyles[0] := Style;
  RVStyle1.TextStyles[1] := Style;
  RVStyle1.TextStyles[0].Color := clWindowText;
  RVStyle1.TextStyles[1].Color := clRed;
  RVStyle1.TextStyles[1].Style := RVStyle1.TextStyles[1].Style + [fsBold];
  FillRVEdit;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVTntSpellWord.rvePaste(Sender: TCustomRichViewEdit;
  var DoDefault: Boolean);
var s: TRVRawByteString;
    ws: WideString;
    i: Integer;
begin
  if not RVGetTextFromClipboardW(s) then
    exit;
  ws := RVU_RawUnicodeToWideString(s);
  for i := 1 to Length(ws) do
    if (ws[i]=#13) or (ws[i]=#10) then
      exit;
  Sender.InsertTextW(ws, False);
  DoDefault := False;
end;
{------------------------------------------------------------------------------}
function TfrmRVTntSpellWord.GetCorrectedText: WideString;
begin
  if FReplacingItemText then
    Result := GetAllText(rve)
  else if lst.ItemIndex>=0 then
    Result := lst.Items[lst.ItemIndex]
  else
    Result := '';
end;
{------------------------------------------------------------------------------}
procedure TfrmRVTntSpellWord.FillRVEdit;
var ErrorItemNo: Integer;
begin
  rve.Clear;
  ErrorItemNo := 0;
  if FStart>1 then begin
    rve.AddNLWTag(Copy(FText, 1, FStart-1), 0, 0, RVEMPTYTAG);
    inc(ErrorItemNo);
  end;
  rve.AddNLWTag(Copy(FText, FStart, FLength), 1, -1, RVEMPTYTAG);
  rve.AddNLWTag(Copy(FText, FStart+FLength, Length(FText)), 0, -1, RVEMPTYTAG);
  rve.Format;
  rve.SetSelectionBounds(ErrorItemNo, FLength+1, ErrorItemNo, FLength+1);
  FReplacingItemText := False;
  UpdateControlsStates;
  btnChange.Default := False;  
  btnIgnore.Default := True;
end;
{------------------------------------------------------------------------------}
function TfrmRVTntSpellWord.IsReplacingItemText: Boolean;
begin
  Result := FReplacingItemText;
end;
{------------------------------------------------------------------------------}
function TfrmRVTntSpellWord.GetStringsForSuggestions: TTntStrings;
begin
  Result := lst.Items;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVTntSpellWord.FormActivate(Sender: TObject);
begin
  rve.SetFocus;
  btnUndoEdit.Default := True;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVTntSpellWord.lstEnter(Sender: TObject);
begin
  btnIgnore.Default := False;
  btnChange.Default := True;
end;
{------------------------------------------------------------------------------}
procedure TfrmRVTntSpellWord.lstDblClick(Sender: TObject);
begin
  ModalResult := btnChange.ModalResult;
end;

end.
