
{*******************************************************}
{                                                       }
{       RichView                                        }
{       TRVWordEnumerator: basic component for          }
{       spell-checkers (or other tools processing       }
{       RichViewEdit word by word)                      }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************}

unit RVWordEnum;

interface

uses Windows, SysUtils, Classes, Controls, Forms, Dialogs, Graphics, RVScroll,
     RichView, RVStyle, RVEdit, CRVData, CRVFData, RVItem, RVTypes, RVThread,
     RVUni;

type
  TRVEnumScope = (rvesFromCursor, rvesFromStart);

  TRVWordEnumerator = class (TPersistent)
    private
      FOldOnCloseQuery: TCloseQueryEvent;
      FWorking: Boolean;
      FCounter: Integer;
      FUnicode: Boolean;
      procedure GetStartLocation(var ItemNo, Offs: Integer;
        var RVData: TCustomRVFormattedData;
        Editor: TCustomRichViewEdit; Scope: TRVEnumScope);
      procedure DoOnCloseQuery(Sender: TObject; var CanClose: Boolean);
      procedure Work(RVData: TCustomRVFormattedData;
        ItemNo, Offs: Integer; Local: Boolean);
      procedure AdjustToWord(var ItemNo, Offs: Integer;
        RVData: TCustomRVFormattedData);
    protected
      FDialogShown: Boolean;
      FRVData: TCustomRVFormattedData;
      FItemNo, FStartOffs, FEndOffs: Integer;
      FWordA, FTextA: TRVAnsiString;    // used if Unicode = False
      FWordW, FTextW: TRVUnicodeString; // used if Unicode = True
      FFontStyle: TFontInfo;
      FWordChanged, FWordStyleChanged, FTextChanged: Boolean;
      FCounterMax: Integer;
      function ProcessWord: Boolean; virtual;
      procedure SelectCurrentWord;
      procedure Run(Editor: TCustomRichViewEdit; Scope: TRVEnumScope);
      procedure RunRVData(Editor: TCustomRichViewEdit; RVData: TCustomRVFormattedData;
        ItemNo, Offs: Integer);
      function CanProcessTextItem(RVData: TCustomRVFormattedData; ItemNo: Integer): Boolean; virtual;
    public
      constructor Create;
      property Working: Boolean read FWorking;
      property Unicode: Boolean read FUnicode write FUnicode;
      property DialogShown: Boolean read FDialogShown;
  end;

implementation

{ TRVWordEnumerator }
{------------------------------------------------------------------------------}
constructor TRVWordEnumerator.Create;
begin
  inherited;
  FCounterMax := 500;
end;
{------------------------------------------------------------------------------}
function TRVWordEnumerator.ProcessWord: Boolean;
{
var s: String;
}
begin
  Result := True;
  {
  // Testing
  if Unicode then begin
    if (FWordW<>'') and (FWordW[1]='a') then begin
      SelectCurrentWord;
      s := FWordW;
      FWordChanged := InputQuery('Test', 'Test:', s);
      FWordW := s;
      Result := FWordChanged;
    end;
    end
  else begin
    if (FWordA<>'') and (FWordA[1]='a') then begin
      SelectCurrentWord;
      s := FWordA;
      FWordChanged := InputQuery('Test', 'Test:', s);
      FWordA := s;
      Result := FWordChanged;
    end;
  end;
  }
end;
{------------------------------------------------------------------------------}
procedure TRVWordEnumerator.SelectCurrentWord;
begin
  FRVData := TCustomRVFormattedData(FRVData.Edit);
  FRVData.SetSelectionBounds(FItemNo, FStartOffs, FItemNo, FEndOffs);
  FRVData.Refresh;
end;
{------------------------------------------------------------------------------}
procedure TRVWordEnumerator.GetStartLocation(var ItemNo, Offs: Integer;
  var RVData: TCustomRVFormattedData;
  Editor: TCustomRichViewEdit; Scope: TRVEnumScope);
var i2, o2: Integer;
begin
  case Scope of
    rvesFromCursor:
      begin
        RVData := Editor.TopLevelEditor.RVData;
        RVData.GetSelectionBoundsEx(ItemNo, Offs, i2, o2, True);
        if (ItemNo<RVData.ItemCount-1) and
           (Offs>=RVData.GetOffsAfterItem(ItemNo)) then begin
          inc(ItemNo);
          Offs := RVData.GetOffsBeforeItem(ItemNo);
        end;
        if (Editor.InplaceEditor=nil) and
           (ItemNo=RVData.ItemCount-1) and
           (Offs>=RVData.GetOffsAfterItem(ItemNo)) then
          GetStartLocation(ItemNo, Offs, RVData, Editor, rvesFromStart);
      end;
    else
      begin
        RVData := Editor.RVData;
        ItemNo := 0;
        Offs := RVData.GetOffsBeforeItem(ItemNo);
      end;
  end;
end;
{------------------------------------------------------------------------------}
function IsDelimiterA(RVData: TCustomRVFormattedData;
  const s: TRVAnsiString; CodePage: TRVCodePage; Index: Integer): Boolean;
begin
  if RichViewApostropheInWord and ((s[Index]='''') or (s[Index]=#146)) then
    Result := (Index=1) or (Index=Length(s)) or
      RVData.IsDelimiterA(s[Index+1], CodePage) or (s[Index+1]='&') or
      RVData.IsDelimiterA(s[Index-1], CodePage) or (s[Index-1]='&')
  else
    Result := RVData.IsDelimiterA(s[Index], CodePage) or (s[Index]='&');  
end;
{------------------------------------------------------------------------------}
function IsDelimiterW(RVData: TCustomRVFormattedData;
  const s: TRVUnicodeString; Index: Integer): Boolean;
begin
  if RichViewApostropheInWord and ((s[Index]='''') or (s[Index]=#146)) then
    Result := (Index=1) or (Index=Length(s)) or
      RVData.IsDelimiterW(s[Index+1]) or (s[Index+1]='&') or
      RVData.IsDelimiterW(s[Index-1]) or (s[Index-1]='&')
  else
    Result := RVData.IsDelimiterW(s[Index]) or (s[Index]='&');  
end;
{------------------------------------------------------------------------------}
procedure TRVWordEnumerator.Work(RVData: TCustomRVFormattedData; ItemNo,
  Offs: Integer; Local: Boolean);
var StoreSub: TRVStoreSubRVData;
    RVStyle: TRVStyle;
   {...................................................}
   function GetFirstDifCharA(const s1, s2: TRVAnsiString; CodePage: TRVCodePage): Integer;
   var i, Len: Integer;
   begin
     Result := Length(s1);
     if Length(s2)<Result then
       Result := Length(s2);
     Len := Result;
     inc(Result);
     for i := 1 to Result-1 do
       if s1[i]<>s2[i] then begin
         Result := i;
         break;
       end;
     if Result<=Len then 
       while (Result>1) and not IsDelimiterA(RVData, s1, CodePage, Result) do
         dec(Result);
   end;
   {...................................................}
   function GetFirstDifCharW(const s1, s2: TRVUnicodeString): Integer;
   var i, Len: Integer;
   begin
     Result := Length(s1);
     if Length(s2)<Result then
       Result := Length(s2);
     Len := Result;
     inc(Result);
     for i := 1 to Result-1 do
       if s1[i]<>s2[i] then begin
         Result := i;
         break;
       end;
     if Result<=Len then 
       while (Result>1) and not IsDelimiterW(RVData, s1, Result) do
         dec(Result);
   end;
   {...................................................}
    function ProcessWord_(var RVData: TCustomRVFormattedData;
     var ItemNo, StartOffs, EndOffs: Integer;
     var sA: TRVAnsiString; var sW: TRVUnicodeString;
     var NewOffs: Integer): Boolean;
    var Len, PrevItemCount: Integer;
      CodePage: TRVCodePage;
    begin
      FRVData := RVData;
      FItemNo := ItemNo;
      FStartOffs := StartOffs;
      FEndOffs := EndOffs;
      Len := EndOffs-StartOffs;
      CodePage := RVData.GetItemCodePage(ItemNo);
      if not Unicode then begin
        FWordA := Copy(sA, StartOffs, Len);
        FTextA := sA;
        end
      else begin
        FWordW := Copy(sW, StartOffs, Len);
        FTextW := sW;
      end;
      FTextChanged := False;
      FWordChanged := False;
      FWordStyleChanged := False;
      FFontStyle := RVStyle.TextStyles[RVData.GetItemStyle(ItemNo)];
      PrevItemCount := RVData.ItemCount;      
      Result := ProcessWord;
      if not Result then
        exit;
      RVData := TCustomRVFormattedData(FRVData.GetRVData);
      if FTextChanged then begin
        RVData.SetSelectionBounds(ItemNo, RVData.GetOffsBeforeItem(ItemNo),
          ItemNo, RVData.GetOffsAfterItem(ItemNo));
        {
        if not Unicode then
          if FTextA='' then
            TCustomRichViewEdit(RVData.GetParentControl).DeleteSelection
          else begin
            TCustomRichViewEdit(RVData.GetParentControl).TopLevelEditor.SetItemTextEdA(ItemNo, FTextA)
          end;
        else
          if FTextW='' then
            TCustomRichViewEdit(RVData.GetParentControl).DeleteSelection
          else begin
            TCustomRichViewEdit(RVData.GetParentControl).TopLevelEditor.SetItemTextEdW(ItemNo, FTextW);
          end;
        }
        if not Unicode then
          TCustomRichViewEdit(RVData.GetParentControl).InsertTextA(FTextA)
        else
          TCustomRichViewEdit(RVData.GetParentControl).InsertTextW(FTextW)
        end
      else if FWordChanged then begin
        RVData.SetSelectionBounds(ItemNo,StartOffs,ItemNo,EndOffs);
        if not Unicode then
          TCustomRichViewEdit(RVData.GetParentControl).InsertTextA(FWordA)
        else
          TCustomRichViewEdit(RVData.GetParentControl).InsertTextW(FWordW)
      end;
      if (FWordChanged or FWordStyleChanged or FTextChanged) and
         (RVData.ItemCount<>PrevItemCount) then begin
        ItemNo := TCustomRichViewEdit(RVData.GetParentControl).TopLevelEditor.CurItemNo;
        NewOffs := TCustomRichViewEdit(RVData.GetParentControl).TopLevelEditor.OffsetInCurItem-1;
        end
      else if FTextChanged then begin
        if not Unicode then
          NewOffs := GetFirstDifCharA(FTextA, sA, CodePage)
        else
          NewOffs := GetFirstDifCharW(FTextW, sW);
        if NewOffs>StartOffs then
          NewOffs := StartOffs;
        dec(NewOffs);
        end
      else begin
        if not Unicode then
          NewOffs := StartOffs+Length(FWordA)
        else
          NewOffs := StartOffs+Length(FWordW);
      end;
      if Unicode then begin
        if FWordStyleChanged or FWordChanged or FTextChanged then begin
          if ItemNo<RVData.ItemCount then
            sW := RVData.GetItemTextW(ItemNo)
          else
            sW := '';
        end;
        end
      else begin
        if FWordStyleChanged or FWordChanged or FTextChanged then begin
          if ItemNo<RVData.ItemCount then
            sA := RVData.GetItemTextA(ItemNo)
          else
            sA := '';
        end;
      end;
      dec(FCounter);
      if FCounter=0 then begin
        Application.ProcessMessages;
        FCounter := FCounterMax;
      end;
    end;
   {...................................................}
   function ProcessText(var RVData: TCustomRVFormattedData; var ItemNo: Integer;
     StartOffs: Integer): Boolean;
   var sA: TRVAnsiString;
       sW: TRVUnicodeString;
       sstart, send: Integer;
       j: Integer;
       CodePage: TRVCodePage;
   begin
      if not CanProcessTextItem(RVData, ItemNo) then begin
        Result := True;
        exit;
      end;
      sstart := 0;
      j := StartOffs;
      if Unicode then begin
        sW := RVData.GetItemTextW(ItemNo);
        sA := '';
        while j<=Length(sW)+1 do begin
          if (j<=Length(sW)) and not IsDelimiterW(RVData, sW, j) then begin
            if sstart = 0 then
              sstart := j
            end
          else if sstart<>0 then begin
            send := j;
            Result := ProcessWord_(RVData, ItemNo, sstart, send, sA, sW, j);
            if not Result then
              exit;
            sstart := 0;
          end;
          inc(j);
        end;
        Result := True;
        end
      else begin
        sA := RVData.GetItemTextA(ItemNo);
        sW := '';
        CodePage := RVData.GetItemCodePage(ItemNo);
        while j<=Length(sA)+1 do begin
          if (j<=Length(sA)) and not IsDelimiterA(RVData, sA, CodePage, j) then begin
            if sstart = 0 then
              sstart := j
            end
          else if sstart<>0 then begin
            send := j;
            Result := ProcessWord_(RVData, ItemNo, sstart, send, sA, sW, j);
            if not Result then
              exit;
            sstart := 0;
          end;
          inc(j);
        end;
        Result := True;
      end;
   end;
   {...................................................}
   function ProcessRVData(RVData: TCustomRVFormattedData; FirstItemNo: Integer): Boolean; forward;
   {...................................................}
   function ProcessItem(Item: TCustomRVItemInfo; ParentRVData: TCustomRVData;
     StoreSub: TRVStoreSubRVData): Boolean;
   var RVData: TCustomRVFormattedData;
   begin
     Result := True;
     if TRVNonTextItemInfo(Item).Hidden and not (rvoShowHiddenText in ParentRVData.Options) then
       exit;
     if StoreSub=nil then
       RVData := TCustomRVFormattedData(item.GetSubRVData(StoreSub, rvdFirst))
     else
       RVData := TCustomRVFormattedData(item.GetSubRVData(StoreSub, rvdNext));
     if RVData<>nil then begin
       repeat
         Result := ProcessRVData(TCustomRVFormattedData(RVData.GetRVData), 0);
         if not Result then
           break;
         RVData := TCustomRVFormattedData(item.GetSubRVData(StoreSub, rvdNext));
       until RVData=nil;
     end;
     StoreSub.Free;
   end;
   {...................................................}
   function ProcessRVData(RVData: TCustomRVFormattedData; FirstItemNo: Integer): Boolean;
   var i: Integer;
   begin
     Result := True;
     i := FirstItemNo;
     while i<RVData.ItemCount do begin
       if RVData.GetItem(i).StyleNo>=0 then
         Result := ProcessText(RVData, i, 1)
       else
         Result := ProcessItem(RVData.GetItem(i), RVData, nil);
       if not Result then
         exit;
       inc(i);
     end;
   end;
   {...................................................}
var r: Boolean;
begin
  RVStyle := RVData.GetRVStyle;
  if (Offs>=RVData.GetOffsAfterItem(ItemNo)) then begin
    r := ItemNo<RVData.ItemCount-1;
    if not r then
      exit;
    inc(ItemNo);
    Offs := RVData.GetOffsBeforeItem(ItemNo);
  end;
  if RVData.GetItemStyle(ItemNo)>=0 then begin
    AdjustToWord(ItemNo, Offs, RVData);
    r := ProcessText(RVData, ItemNo, Offs);
    if not r then
      exit;
    Inc(ItemNo);
  end;

  while RVData<>nil do begin
    ProcessRVData(RVData, ItemNo);
    if Local then
      exit;
    RVData.GetParentInfo(ItemNo, StoreSub);
    if ItemNo<0 then
      break;
    RVData := TCustomRVFormattedData(RVData.GetAbsoluteParentData);
    r := ProcessItem(RVData.GetItem(ItemNo), RVData, StoreSub);
    if not r then
      break;
    inc(ItemNo);
  end;
end;
{------------------------------------------------------------------------------}
procedure TRVWordEnumerator.Run(Editor: TCustomRichViewEdit;
  Scope: TRVEnumScope);
var ItemNo, Offs: Integer;
    RVData : TCustomRVFormattedData;
    Form: TWinControl;
begin
  FDialogShown := False;
  Form := Editor;
  while Form.Parent<>nil do begin
    Form := Form.Parent;
    if Form is TForm then
      break;
  end;
  if Form is TForm then begin
    FOldOnCloseQuery := TForm(Form).OnCloseQuery;
    TForm(Form).OnCloseQuery := DoOnCloseQuery;
    end
  else
    Form := nil;
  FCounter := FCounterMax;
  FWorking := True;
  try
    GetStartLocation(ItemNo, Offs, RVData, Editor, Scope);
    Work(RVData, ItemNo, Offs, False);
  finally
    FWorking := False;
    if Form<>nil then
      TForm(Form).OnCloseQuery := FOldOnCloseQuery;
    FOldOnCloseQuery := nil;
  end;
end;
{------------------------------------------------------------------------------}
procedure TRVWordEnumerator.RunRVData(Editor: TCustomRichViewEdit;
  RVData: TCustomRVFormattedData; ItemNo, Offs: Integer);
var
  Form: TWinControl;
begin
  FDialogShown := False;
  Form := Editor;
  while Form.Parent<>nil do begin
    Form := Form.Parent;
    if Form is TForm then
      break;
  end;
  if Form is TForm then begin
    FOldOnCloseQuery := TForm(Form).OnCloseQuery;
    TForm(Form).OnCloseQuery := DoOnCloseQuery;
    end
  else
    Form := nil;
  FCounter := FCounterMax;
  FWorking := True;
  try
    Work(RVData, ItemNo, Offs, True);
  finally
    FWorking := False;
    if Form<>nil then
      TForm(Form).OnCloseQuery := FOldOnCloseQuery;
    FOldOnCloseQuery := nil;
  end;
end;
{------------------------------------------------------------------------------}
procedure TRVWordEnumerator.DoOnCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  Beep;
  CanClose := False;
end;
{------------------------------------------------------------------------------}
procedure TRVWordEnumerator.AdjustToWord(var ItemNo, Offs: Integer;
  RVData: TCustomRVFormattedData);
var sA: TRVAnsiString;
    sW: TRVUnicodeString;
    CodePage: TRVCodePage;
begin
  if (RVData.GetItemStyle(ItemNo)>=0) and
     CanProcessTextItem(RVData, ItemNo) then begin
    if Unicode then begin
      sW := RVData.GetItemTextW(ItemNo);
      while (Offs>1) and not IsDelimiterW(RVData, sW, Offs) do
        dec(Offs);
      end
    else begin
      sA := RVData.GetItemTextA(ItemNo);
      CodePage := RVData.GetItemCodePage(ItemNo);
      while (Offs>1) and not IsDelimiterA(RVData, sA, CodePage, Offs) do
        dec(Offs);
    end;
  end;
end;
{------------------------------------------------------------------------------}
function TRVWordEnumerator.CanProcessTextItem(RVData: TCustomRVFormattedData;
  ItemNo: Integer): Boolean;
var TextStyle: TFontInfo;
begin
  TextStyle := RVData.GetRVStyle.TextStyles[RVData.GetItemStyle(ItemNo)];
  Result := TextStyle.Charset<>SYMBOL_CHARSET;
  if Result and not (rvoShowHiddenText in RVData.Options) then
    Result := not (rvteoHidden in TextStyle.Options);
end;

end.
